// MyRobot.c AGBmain Entry point is contained here (main program loop)

#include "gba.h"
#include "userdefs.h"
#include "rand.h"

  // GBA Headers
#include "gfx.h"
#include "sprite.h"
#include "input.h"
#include "mathluts.h"

  // Game Headers
#include "gamedefs.h"
#include "intro.h"
#include "sound.h"
#include "blit.h"
#include "enemyCode.h"
#include "robotShotCode.h"
#include "battery.h"

  // Images
#include "imagesHud.h"
#include "imagesSprite.h"
#include "imagesNonTiled.h"
#include "lcdPalette.h"

cheatTable cheats;

int score;
int shouldReset=0;
int isGameRunning;

u16 fgColor;
u16 bgColor;
u16 bgfColor;

   // Robot Position is also the same as the mover position
int robotPos = ROBOT_INITIAL_POS; // This var is always either 0, 1, 2 or 3
int playerPosX = PLAYER_INIT_POSX;
int playerPosY = PLAYER_INIT_POSY;
int numBatteries = BATTERIES_INITIAL;

img* digitImg[10];

void ResetField(void);
void ResetPlayerElements(void);
void HidePlayerElements(void);

////////////////////////////////////////////

void MapImagesToSprites(void)
{
	int i, row;

	  // Enemy, Robot Shot, and Robot
	for(row=0; row < NUM_ENEMY_ROWS; row++)
	{
		for(i=0; i < NUM_ENEMY_COLS; i++)
		{
			spriteImg[spEnemyFirst     + (row*NUM_ENEMY_COLS) + i] = &img_enemy;
			spriteImg[spRobotShotFirst + (row*NUM_ENEMY_COLS) + i] = &img_robotShot;

			if(row==0)
			{
				spriteImg[spRobotFirst + i] = &img_robot;
				spriteImg[spRobotHeadFirst + i] = &img_robotHead;
			}
		}
	}

	  // Score
	for(i=0; i<NUM_SCORE_DIGITS; i++)
	{
		spriteImg[spScoreFirst + i] = &img_num0;
	}

	  // Player
	for(i=0; i<NUM_PLAYER_COLS; i++)
	{
		spriteImg[spPlayerFirst+i] = &img_playerMiddle;
	}
	for(; i < NUM_PLAYER_COLS*2; i++)
	{
		spriteImg[spPlayerFirst+i] = &img_playerMiddle;
	}
	for(; i < NUM_PLAYER_POS; i++)
	{
		spriteImg[spPlayerFirst+i] = &img_playerBottom;
	}

	  // Mover
	for(i=0; i < NUM_MOVER_SECS; i++)
	{
		spriteImg[spMoverSecFirst   + i] = &img_moverSection;

		if(i<NUM_MOVER_LEFT)
			spriteImg[spMoverLeftFirst  + i] = &img_moverLeft;

		if(i>0)
			spriteImg[spMoverRightFirst + i] = &img_moverRight;
	}

	  // Battery Status
	for(i=0; i < MAX_BATTERIES; i++)
	{
		spriteImg[spBatteryStatFirst + i] = &img_batteryStat;
	}

	  // Miss Status
	for(i=0; i < MAX_MISSES; i++)
	{
		spriteImg[spMissStatFirst + i] = &img_missFull;
	}

	  // Batteries
	spriteImg[spBatteryUpFirst + 0] = &img_battFall3;
	spriteImg[spBatteryUpFirst + 1] = &img_battFall2;
	spriteImg[spBatteryUpFirst + 2] = &img_battFall1;
	spriteImg[spBatteryUpFirst + 3] = &img_battFall;
	spriteImg[spBatteryUpFirst + 4] = &img_battFall5;
	spriteImg[spBatteryUpFirst + 5] = &img_battFall4;
	spriteImg[spBatteryUpFirst + 6] = &img_battFall3;
	spriteImg[spBatteryUpFirst + 7] = &img_battFall2;

	spriteImg[spBatteryLeftFirst + 0] = &img_battLeft5;
	spriteImg[spBatteryLeftFirst + 1] = &img_battLeft4;
	spriteImg[spBatteryLeftFirst + 2] = &img_battLeft3;
	spriteImg[spBatteryLeftFirst + 3] = &img_battLeft2;
	spriteImg[spBatteryLeftFirst + 4] = &img_battLeft1;
	spriteImg[spBatteryLeftFirst + 5] = &img_battLeft;
	spriteImg[spBatteryLeftFirst + 6] = &img_battLeft5;
	spriteImg[spBatteryLeftFirst + 7] = &img_battLeft4;

	spriteImg[spBatteryFallFirst + (0 * NUM_BATTFALL_COLS) + 0] = &img_battFall;
	spriteImg[spBatteryFallFirst + (1 * NUM_BATTFALL_COLS) + 0] = &img_battFall1;
	spriteImg[spBatteryFallFirst + (2 * NUM_BATTFALL_COLS) + 0] = &img_battFall2;
	spriteImg[spBatteryFallFirst + (3 * NUM_BATTFALL_COLS) + 0] = &img_battFall3;
	spriteImg[spBatteryFallFirst + (4 * NUM_BATTFALL_COLS) + 0] = &img_battFall4;

	spriteImg[spBatteryFallFirst + (0 * NUM_BATTFALL_COLS) + 1] = &img_battFall4;
	spriteImg[spBatteryFallFirst + (1 * NUM_BATTFALL_COLS) + 1] = &img_battFall5;
	spriteImg[spBatteryFallFirst + (2 * NUM_BATTFALL_COLS) + 1] = &img_battFall;
	spriteImg[spBatteryFallFirst + (3 * NUM_BATTFALL_COLS) + 1] = &img_battFall1;
	spriteImg[spBatteryFallFirst + (4 * NUM_BATTFALL_COLS) + 1] = &img_battFall2;

	spriteImg[spBatteryFallFirst + (0 * NUM_BATTFALL_COLS) + 2] = &img_battFall2;
	spriteImg[spBatteryFallFirst + (1 * NUM_BATTFALL_COLS) + 2] = &img_battFall3;
	spriteImg[spBatteryFallFirst + (2 * NUM_BATTFALL_COLS) + 2] = &img_battFall4;
	spriteImg[spBatteryFallFirst + (3 * NUM_BATTFALL_COLS) + 2] = &img_battFall5;
	spriteImg[spBatteryFallFirst + (4 * NUM_BATTFALL_COLS) + 2] = &img_battFall;

	spriteImg[spBatteryFallFirst + (0 * NUM_BATTFALL_COLS) + 3] = &img_battFall;
	spriteImg[spBatteryFallFirst + (1 * NUM_BATTFALL_COLS) + 3] = &img_battFall1;
	spriteImg[spBatteryFallFirst + (2 * NUM_BATTFALL_COLS) + 3] = &img_battFall2;
	spriteImg[spBatteryFallFirst + (3 * NUM_BATTFALL_COLS) + 3] = &img_battFall3;
	spriteImg[spBatteryFallFirst + (4 * NUM_BATTFALL_COLS) + 3] = &img_battFall4;

	spriteImg[spBatteryRelease] = &img_battReleaseClosed;
	spriteImg[spBatteryReleaseDoorOpen] = &img_battReleaseDoorOpen;

	  // Zap
	spriteImg[spZap] = &img_zap;

	  // Battery Increase Display
	spriteImg[spBattInc] = &img_battInc;

}

void InitSprites(void)
{
	int i, row;

	for(row=0; row < NUM_ENEMY_ROWS; row++)
	{
		for(i=0; i<NUM_ENEMY_COLS; i++)
		{
			  // Init Enemy Sprite
			Sprite_InitXY(spEnemyFirst + (row*NUM_ENEMY_COLS) + i, SPRITE_4BIT,
				ENEMY_START_X + (i * ENEMY_SPACING_X),
				ENEMY_START_Y + (row * ENEMY_SPACING_Y));

			  // Init RobotShot Sprite
			Sprite_InitXY(spRobotShotFirst + (row*NUM_ENEMY_COLS) + i, SPRITE_4BIT,
				ENEMY_START_X + (i * ENEMY_SPACING_X) + ROBOTSHOT_OFFSET_X,
				ENEMY_START_Y + (row * ENEMY_SPACING_Y) + ROBOTSHOT_OFFSET_Y);

			if(row==0) // Only one row of robots
			{
				  // Init Robot Sprite
				Sprite_InitXY(spRobotFirst + i, SPRITE_4BIT,
					ROBOT_START_X + (i * ROBOT_STEP_X), ROBOT_Y);
				Sprite_InitXY(spRobotHeadFirst + i, SPRITE_4BIT,
					ROBOT_START_X + (i * ROBOT_STEP_X) + ROBOTHEAD_OFFSET_X,
					ROBOT_Y + ROBOTHEAD_OFFSET_Y);
			}
		}
	}

	  // Score
	for(i=0; i<NUM_SCORE_DIGITS; i++)
	{
		Sprite_InitXY(spScoreFirst + i, SPRITE_4BIT,
			SCORE_X - (DIGIT_WIDTH*i), SCORE_Y);
	}

	  // Player
	for(row=0; row < NUM_PLAYER_ROWS; row++)
	{
		for(i=0; i<NUM_PLAYER_COLS; i++)
		{
			Sprite_InitXY(spPlayerFirst + (row*NUM_PLAYER_COLS) + i, SPRITE_4BIT,
				PLAYER_X + (i * PLAYER_STEP_X),
				PLAYER_Y + (row * PLAYER_STEP_Y));
		}
	}

	  // Mover
	for(i=0; i<NUM_MOVER_SECS; i++)
	{
		  // Init MoverSection Sprites
		Sprite_InitXY(spMoverSecFirst + i, SPRITE_4BIT,
			MOVER_X + (i * MOVER_SPACING_X), MOVER_Y);

		  // Init MoverLeft Sprites
		if(i < NUM_MOVER_LEFT)
		{
			Sprite_InitXY(spMoverLeftFirst + i, SPRITE_4BIT,
				MOVER_X + (i * MOVER_SPACING_X), MOVER_Y);
		}

		  // Init MoverRight Sprites
		if(i > 0)
		{
			Sprite_InitXY(spMoverRightFirst + i, SPRITE_4BIT,
				MOVER_X + (i * MOVER_SPACING_X), MOVER_Y);
		}
	}

	  // Battery Status
	for(i=0; i < (MAX_BATTERIES/2); i++)
	{
		  // Init Battery Status Sprites
		Sprite_InitXY(spBatteryStatFirst + i*2, SPRITE_4BIT,
			BATTERYSTAT_X + (i * BATTERYSTAT_SPACING_X),
			BATTERYSTAT_Y + BATTERYSTAT_SPACING_Y);
		LCD_OFF(spBatteryStatFirst + i*2);

		  // Init Battery Status Sprites
		Sprite_InitXY(spBatteryStatFirst + i*2 + 1, SPRITE_4BIT,
			BATTERYSTAT_X + (i * BATTERYSTAT_SPACING_X),
			BATTERYSTAT_Y);
		LCD_OFF(spBatteryStatFirst + i*2 + 1);
	}

	  // Miss Status
	for(i=0; i < MAX_MISSES; i++)
	{
		Sprite_InitXY(spMissStatFirst + i, SPRITE_4BIT,
			MISS_X + (MISS_SPACING_X * i),
			MISS_Y);
	}

	  // Batteries
	for(i=0; i < NUM_BATTUP-1; i++)
	{
		Sprite_InitXY(spBatteryUpFirst + i, SPRITE_4BIT,
			BATTUP_X, BATTUP_START_Y + (i*BATTUP_SPACING_Y));
		LCD_OFF(spBatteryUpFirst + i);
		Sprite_VFlipOn(spBatteryUpFirst + i);
		Sprite_HFlipOn(spBatteryUpFirst + i);
	}
	Sprite_InitXY(spBatteryUpLast, SPRITE_4BIT,
		BATTUP_X, BATTUP_LAST_Y);
	Sprite_VFlipOn(spBatteryUpFirst + i);
	Sprite_HFlipOn(spBatteryUpFirst + i);

	for(i=0; i < NUM_BATTLEFT; i++)
	{
		Sprite_InitXY(spBatteryLeftFirst + i, SPRITE_4BIT,
			BATTLEFT_START_X + (BATTLEFT_SPACING_X * i),
			BATTLEFT_Y);
		LCD_OFF(spBatteryLeftFirst + i);
	}
	for(row=0; row<NUM_BATTFALL_ROWS; row++)
	{
		for(i=0; i < NUM_BATTFALL_COLS; i++)
		{
			Sprite_InitXY(spBatteryFallFirst + i + (row*NUM_BATTFALL_COLS), SPRITE_4BIT,
				BATTFALL_START_X + (BATTFALL_SPACING_X * i),
				BATTFALL_START_Y + (BATTFALL_SPACING_Y * row));
			LCD_OFF(spBatteryFallFirst + i + (row*NUM_BATTFALL_COLS));
		}
	}
	Sprite_InitXY(spBatteryRelease, SPRITE_4BIT, BATTRELEASE_X, BATTRELEASE_Y);
	Sprite_InitXY(spBatteryReleaseDoorOpen, SPRITE_4BIT,
		BATTRELEASEDOOROPEN_X, BATTRELEASEDOOROPEN_Y);
	LCD_OFF(spBatteryReleaseDoorOpen);

	Sprite_InitXY(spZap, SPRITE_4BIT, ZAP_X, ZAP_Y);
	LCD_OFF(spZap);

	Sprite_InitXY(spBattInc, SPRITE_4BIT, BATTINC_X, BATTINC_Y);
	LCD_OFF(spBattInc);
}

void LoadImages(void)
{
	int i;

	LoadSpriteImage(&img_playerBottom, SPRITE_4BIT);
	LoadSpriteImage(&img_playerMiddle, SPRITE_4BIT);
	LoadSpriteImage(&img_playerMiddleShoot, SPRITE_4BIT);
	LoadSpriteImage(&img_enemy, SPRITE_4BIT);
	LoadSpriteImage(&img_robot, SPRITE_4BIT);
	LoadSpriteImage(&img_robotHead, SPRITE_4BIT);
	LoadSpriteImage(&img_robotTrack, SPRITE_4BIT);
	LoadSpriteImage(&img_robotStandEmpty, SPRITE_4BIT);
	LoadSpriteImage(&img_robotLCDTest, SPRITE_4BIT);
	LoadSpriteImage(&img_robotShot, SPRITE_4BIT);
	for(i=0; i<10; i++)
		LoadSpriteImage(digitImg[i], SPRITE_4BIT);
	LoadSpriteImage(&img_moverSection, SPRITE_4BIT);
	LoadSpriteImage(&img_moverLeft, SPRITE_4BIT);
	LoadSpriteImage(&img_moverRight, SPRITE_4BIT);
	LoadSpriteImage(&img_batteryStat, SPRITE_4BIT);

	LoadSpriteImage(&img_missEmpty, SPRITE_4BIT);
	LoadSpriteImage(&img_missFull, SPRITE_4BIT);

	LoadSpriteImage(&img_battLeft, SPRITE_4BIT);
	LoadSpriteImage(&img_battFall, SPRITE_4BIT);
	LoadSpriteImage(&img_battLeft1, SPRITE_4BIT);
	LoadSpriteImage(&img_battFall1, SPRITE_4BIT);
	LoadSpriteImage(&img_battLeft2, SPRITE_4BIT);
	LoadSpriteImage(&img_battFall2, SPRITE_4BIT);
	LoadSpriteImage(&img_battLeft3, SPRITE_4BIT);
	LoadSpriteImage(&img_battFall3, SPRITE_4BIT);
	LoadSpriteImage(&img_battLeft4, SPRITE_4BIT);
	LoadSpriteImage(&img_battFall4, SPRITE_4BIT);
	LoadSpriteImage(&img_battLeft5, SPRITE_4BIT);
	LoadSpriteImage(&img_battFall5, SPRITE_4BIT);

	LoadSpriteImage(&img_battReleaseClosed, SPRITE_4BIT);
	LoadSpriteImage(&img_battReleaseOpen, SPRITE_4BIT);
	LoadSpriteImage(&img_battReleaseLCDTest, SPRITE_4BIT);
	LoadSpriteImage(&img_battReleaseDoorOpen, SPRITE_4BIT);

	LoadSpriteImage(&img_zap, SPRITE_4BIT);
	LoadSpriteImage(&img_battInc, SPRITE_4BIT);
}

void ResetPlayer(void)
{
	int i;
	playerPosX = PLAYER_INIT_POSX;
	playerPosY = PLAYER_INIT_POSY;

	for(i=0; i<NUM_PLAYER_POS; i++)
	{
		LCD_OFF(spPlayerFirst + i);
	}

	LCD_ON(spPlayerFirst + PLAYER_INIT_POSX + (NUM_PLAYER_COLS * PLAYER_INIT_POSY));
}

void DrawRobotTrack(int pos)
{
	int i;
//	int x, y;
	for(i=0; i<4; i++)
	{

		if(i==pos)
		{
			Sprite_SetCharacter(spRobotFirst + i, &img_robot);
			LCD_ON(spRobotHeadFirst + i);
			  // Draw Head
//			Blit(&img_robotHead, fgColor, bgColor,
//				Sprite_GetX(spRobotFirst + i) + ROBOTHEAD_OFFSET_X,
//				Sprite_GetY(spRobotFirst + i) + ROBOTHEAD_OFFSET_Y);
		}
		else
		{
			Sprite_SetCharacter(spRobotFirst + i, &img_robotTrack);
			LCD_OFF(spRobotHeadFirst + i);

/*			  // Erase Head
			x = Sprite_GetX(spRobotFirst + i) + ROBOTHEAD_OFFSET_X;
			y = Sprite_GetY(spRobotFirst + i) + ROBOTHEAD_OFFSET_Y;

			DrawHLineLength(bgColor, y + 0, x, ROBOTHEAD_WIDTH);
			DrawHLineLength(bgColor, y + 1, x, ROBOTHEAD_WIDTH);
			DrawHLineLength(bgColor, y + 2, x, ROBOTHEAD_WIDTH);
			DrawHLineLength(bgColor, y + 3, x, ROBOTHEAD_WIDTH);
			DrawHLineLength(bgColor, y + 4, x, ROBOTHEAD_WIDTH);
*/
		}
	}
}

void SetMoverPos(int pos)
{
	  // Set them all to MoverSection
	int i;
	for(i=0; i<NUM_MOVER_SECS; i++)
	{
		LCD_ON (spMoverSecFirst   + i);

		if(i<NUM_MOVER_LEFT)
			LCD_OFF(spMoverLeftFirst  + i);

		if(i > 0)
			LCD_OFF(spMoverRightFirst + i);
	}

	  // Left of pos
	if(pos > 0)
	{
		LCD_OFF(spMoverSecFirst + pos-1);
		LCD_ON(spMoverLeftFirst + pos-1);
	}

	  // Pos
	LCD_OFF(spMoverSecFirst + pos);

	  // Right of pos
	LCD_OFF(spMoverSecFirst  + pos+1);
	LCD_ON(spMoverRightFirst + pos+1);

	  // Robot Track Lines
	DrawRobotTrack(pos);
}

void ResetRobot(void)
{
	int i;
	robotPos = ROBOT_INITIAL_POS;

	for(i=0; i<NUM_PLAYER_COLS; i++)
	{
		Sprite_SetCharacter(spRobotFirst+i, &img_robotTrack);
		LCD_ON(spRobotFirst + i);

		LCD_OFF(spRobotHeadFirst + i);
	}

	Sprite_SetCharacter(spRobotFirst + ROBOT_INITIAL_POS, &img_robot);
	LCD_ON(spRobotHeadFirst + ROBOT_INITIAL_POS);
}

void ResetMover(void)
{
	SetMoverPos(ROBOT_INITIAL_POS);
}

void DisplayBatteryStat(int numBatteries)
{
	int i;

	if(numBatteries < 0)
		return;

	for(i=0; (i<numBatteries) && (i<MAX_BATTERIES); i++)
	{
		LCD_ON(spBatteryStatFirst + i);
	}

	for(; i<MAX_BATTERIES; i++)
	{
		LCD_OFF(spBatteryStatFirst + i);
	}
}

void ResetBatteryStat(void)
{
	numBatteries = BATTERIES_INITIAL;

	if(cheats.fullBatteries)
		numBatteries = 8;

	DisplayBatteryStat(numBatteries);
}

int inline IsBetween(s16 val, s16 min, s16 max)
{
	return (val > min) && (val <= max);
}

int AreOverlapping(u16 sprite1, u16 sprite2)
{
	// Pretend we extend the box s2 by s1_w on the right
	// and by s1_h on the bottom.  If the lower-right
	// corner of the box s1 is within the extended s2,
	// then there is a collision;

	s16 s1_width  = spriteImg[sprite1]->width;
	s16 s1_height = spriteImg[sprite1]->height;
	s16 s1_right  = Sprite_GetX(sprite1) + s1_width;
	s16 s1_bottom = Sprite_GetY(sprite1) + s1_height;

	  // s2e: Box s2 Extended
	s16 s2e_left = Sprite_GetX(sprite2);
	s16 s2e_top  = Sprite_GetY(sprite2);
	s16 s2e_right  = s2e_left   + spriteImg[sprite2]->width  + s1_width;
	s16 s2e_bottom = s2e_top + spriteImg[sprite2]->height;// + s1_height;

	return (IsBetween(s1_right, s2e_left, s2e_right)  &&
			IsBetween(s1_bottom, s2e_top, s2e_bottom) );
}

void DrawFgLines(u16 color)
{
//	Blit(&img_consolefever, bgfColor, bgColor,
//		CONSOLEFEVER_GAME_X, CONSOLEFEVER_GAME_Y);
	Blit(&img_myRobotText, bgfColor, bgColor,
		MYROBOTTEXT_X, MYROBOTTEXT_Y);
	Blit(&img_scoreText, bgfColor, bgColor,
		SCORETEXT_X, SCORETEXT_Y);
	Blit(&img_batteryText, bgfColor, bgColor,
		BATTERYTEXT_X, BATTERYTEXT_Y);
	Blit(&img_missText, bgfColor, bgColor,
		MISSTEXT_X, MISSTEXT_Y);

	DrawHLineLength(bgfColor, 159-24, 0, 240);
	DrawVLineLength(bgfColor, 134, 0, 160);
	DrawVLineToFrom(bgfColor, 132, 0, 135);
	DrawHLineToFrom(bgfColor, 112, 106, 131);

	  // Around the Battery Release
	DrawVLineToFrom(bgfColor, 106, 20, 77);
	DrawVLineToFrom(bgfColor, 106, 92, 112);
	DrawHLineToFrom(bgfColor, 78, 105, 111);
	DrawHLineToFrom(bgfColor, 91, 105, 111);
	DrawVLineToFrom(bgfColor, 111, 79, 90);

	  // Dotted Line on Robot Side
	DrawHLineToFrom(bgfColor, 93, 135, 141);
	DrawHLineLength(bgfColor, 93, 157, 10);
	DrawHLineLength(bgfColor, 93, 182, 10);
	DrawHLineLength(bgfColor, 93, 207, 10);
	DrawHLineToFrom(bgfColor, 93, 232, 239);

	  // Battery Falls: Lower
	DrawHLineToFrom(bgfColor, 72, 0, 2);
	DrawHLineLength(bgfColor, 72, 22+2, 3);
	DrawHLineLength(bgfColor, 72, 48+2, 3);
	DrawHLineLength(bgfColor, 72, 74+2, 3);
	DrawHLineToFrom(bgfColor, 72, 103, 106);

	  // Battery Falls: Upper
	DrawHLineToFrom(bgfColor, 20, 0, 2);
	DrawHLineLength(bgfColor, 20, 22+2, 3);
	DrawHLineLength(bgfColor, 20, 48+2, 3);
	DrawHLineLength(bgfColor, 20, 74+2, 3);
	DrawHLineToFrom(bgfColor, 20, 103, 106);

	  // Battery Falls: Vertical
	DrawVLineToFrom(bgfColor, 2, 20, 72);
	DrawVLineToFrom(bgfColor, 22+2,   20, 72);
	DrawVLineToFrom(bgfColor, 22+2+3, 20, 72);
	DrawVLineToFrom(bgfColor, 48+2,   20, 72);
	DrawVLineToFrom(bgfColor, 48+2+3, 20, 72);
	DrawVLineToFrom(bgfColor, 74+2,   20, 72);
	DrawVLineToFrom(bgfColor, 74+2+3, 20, 72);
	DrawVLineToFrom(bgfColor, 103,  20, 72);

	  // Lines In Status Bar
	DrawVLineToFrom(bgfColor, 199, 136, 159);
	DrawVLineToFrom(bgfColor, 57,  136, 159);

	//DrawRectSize(bgfColor, 0, 0, 240, 160);
}

void SetFgColor(u16 color)
{
	fgColor = color;
	SetSpritePaletteEntry8Bit(FG_COLOR_INDEX, color);
	DrawFgLines(color);
}

u16 GetFgColor(void)
{
	return GetSpritePaletteEntry8Bit(FG_COLOR_INDEX);
}

void UpdateScreen(void)
{
	  // OAM only writable during VBlank
	WaitForVSync();
	Sprite_UpdateOAM(0, endOfSprites);
}

void LCDTestDisplay(void)
{
	int i;
	for(i=0; i < endOfSprites; i++)
	{
		LCD_ON(i);
	}

	for(i = spScoreFirst; i <= spScoreLast; i++)
	{
		Sprite_SetCharacter(i, digitImg[8]);
	}

	for(i = spMissStatFirst; i <= spMissStatLast; i++)
	{
		Sprite_SetCharacter(i, &img_missFull);
	}

	for(i = spRobotFirst; i <= spRobotLast; i++)
	{
		Sprite_SetCharacter(i, &img_robotLCDTest);
	}

	for(i = 0; i < NUM_PLAYER_COLS*2; i++)
	{
		Sprite_SetCharacter(spPlayerFirst+i, &img_playerMiddleShoot);
	}

	Sprite_SetCharacter(spBatteryRelease, &img_battReleaseLCDTest);
	Blit(&img_paused, fgColor, bgColor, PAUSEDTEXT_X, PAUSEDTEXT_Y);
	Blit(&img_pressStart, fgColor, bgColor, PRESSSTARTTEXT_X, PRESSSTARTTEXT_Y);
	Blit(&img_gameover, fgColor, bgColor, GAMEOVERTEXT_X, GAMEOVERTEXT_Y);
}

void HandleReset(void)
{
	if(isKeyDown(KEY_L) && isKeyDown(KEY_R))
	{
		  // Go into LCD test display
		LCDTestDisplay();
		UpdateScreen();

		  // Wait until L and R are both released
		do{ Input_Poll(); }
		while(isKeyDown(KEY_L) || isKeyDown(KEY_R));

		shouldReset=1;
	}
}

void HandleColorSwitch(void)
{
	if(isKeyPressed(KEY_SELECT))
	{
		if(fgColor == FG_COLOR_0)
		{
			fgColor = FG_COLOR_1;
			bgColor = BG_COLOR_1;
			bgfColor = BGF_COLOR_1;
		}
		else
		{
			fgColor = FG_COLOR_0;
			bgColor = BG_COLOR_0;
			bgfColor = BGF_COLOR_0;
		}

		WaitForVSync();
		FillScreen(bgColor);
		SetFgColor(fgColor);
	}
}

void DoPause(void)
{
	int delayTimer=0;
	int startReleased=0;
	int done=0;
	int isTextOn=1;
	  
	WaitForVSync();
	Blit(&img_paused, fgColor, bgColor, PAUSEDTEXT_X, PAUSEDTEXT_Y);
	UpdateScreen();
	do
	{
		if(songEnded)
		{
			StopMusic();
			songEnded = 0;
		}

		delayTimer++;
		WaitForVSync();
		if(delayTimer >= PAUSE_ANIM_DELAY)
		{
			delayTimer=0;
			if(isTextOn)
			{
				Blit(&img_paused, bgColor, bgColor, PAUSEDTEXT_X, PAUSEDTEXT_Y);
				isTextOn=0;
			}
			else
			{
				Blit(&img_paused, fgColor, bgColor, PAUSEDTEXT_X, PAUSEDTEXT_Y);
				isTextOn=1;
			}
		}

		Input_Poll();
		HandleReset();
		if(shouldReset)
			return;

		HandleColorSwitch();

		if(!startReleased)
		{  // Wait until start is released
			if(isKeyReleased(KEY_START))
			{
				  // Short Delay
				WaitForVSync();
				startReleased = 1;
			}
		}
		else
		{  // Continue to wait until pressed again
			if(isKeyPressed(KEY_START))
				done = 1;
		}
	} while(!done);

	Blit(&img_paused, bgColor, bgColor, PAUSEDTEXT_X, PAUSEDTEXT_Y);
}

void LoseTry(u16 guiltySprite)
{
	int delayTimer=0;
	int i;
	int missIndex;
	int isGameOver=0;
//	u16 saveOldFgColor;

//	saveOldFgColor = fgColor;
	if(cheats.unlimitedMisses)
		missIndex = spMissStatFirst;
	else
	{
		for(missIndex=spMissStatFirst; missIndex<=spMissStatLast; missIndex++)
		{
			if(spriteImg[missIndex] == &img_missEmpty)
				break;
		}

		if(missIndex == spMissStatLast)
		{
			// Game Over .........
			isGameOver = 1;
			StartMusic(1); //1=gameOverMelody
		//	SetFgColor(RGB(29, 3, 3));  // No red, it's ugly and cheap-looking
			Blit(&img_gameover, fgColor, bgColor, GAMEOVERTEXT_X, GAMEOVERTEXT_Y);
			Sprite_SetCharacter(missIndex, &img_missFull);
			LCD_ON(guiltySprite);
			UpdateScreen();
			while(!songEnded)
			{
				WaitForVSync();
			}
			StopMusic();
			songEnded = 0;
			Sprite_SetCharacter(missIndex, &img_missEmpty);
		}
		else 
			LCD_TOGGLE(guiltySprite);

	}

//	SetFgColor(RGB(29, 3, 3));  // No red, it's ugly and cheap-looking
	if(isGameOver)
		Blit(&img_gameover, fgColor, bgColor, GAMEOVERTEXT_X, GAMEOVERTEXT_Y);

	for(i=0; i<LOSETRY_NUM_FLASHES; i++)
	{
		LCD_TOGGLE(guiltySprite);
		PlayBeep1();

		if(spriteImg[missIndex] == &img_missEmpty)
			Sprite_SetCharacter(missIndex, &img_missFull);
		else
			Sprite_SetCharacter(missIndex, &img_missEmpty);

		UpdateScreen();

		for(delayTimer=0; delayTimer<LOSETRY_ANIM_DELAY; delayTimer++)
		{
			Input_Poll();
			HandleReset();

			if(shouldReset)
				return;

			WaitForVSync();
		}

		if(songEnded)
		{
			StopMusic();
			songEnded = 0;
		}
	}
	Sprite_SetCharacter(missIndex, &img_missFull);

//	SetFgColor(saveOldFgColor);

	ResetField();

	if(isGameOver)
	{
		Blit(&img_gameover, fgColor, bgColor, GAMEOVERTEXT_X, GAMEOVERTEXT_Y);
		HidePlayerElements();
		isGameRunning=0;
	}
	else
	{
		ResetPlayerElements();
	}

}

void SetScoreDisplay(int score)
{
	int i;
	int digit;
	for(i = spScoreFirst; i <= spScoreLast; i++)
	{
		digit = score % 10;
		Sprite_SetCharacter(i, digitImg[digit]);
		LCD_ON(i);
		score -= digit;
		score /= 10;
	}

	for(i=spScoreLast; spriteImg[i] == digitImg[0] && i >= spScoreFirst+1; i--)
	{
		LCD_OFF(i);
	}
}

void SetupDigitLookup(void)
{
	digitImg[0] = &img_num0;
	digitImg[1] = &img_num1;
	digitImg[2] = &img_num2;
	digitImg[3] = &img_num3;
	digitImg[4] = &img_num4;
	digitImg[5] = &img_num5;
	digitImg[6] = &img_num6;
	digitImg[7] = &img_num7;
	digitImg[8] = &img_num8;
	digitImg[9] = &img_num9;
}

void DoGameNotRunning(void)
{
	int frameCount=0;
	int isTextOn=0;

	do
	{
		if(shouldReset)
			break;

		if(songEnded)
		{
			StopMusic();
			songEnded = 0;
		}

		frameCount++;
		WaitForVSync();
		if(frameCount >= PRESSSTART_ANIM_DELAY)
		{
			frameCount = 0;

			if(isTextOn)
			{
				Blit(&img_pressStart, bgColor, bgColor, PRESSSTARTTEXT_X, PRESSSTARTTEXT_Y);
				isTextOn=0;
			}
			else
			{
				Blit(&img_pressStart, fgColor, bgColor, PRESSSTARTTEXT_X, PRESSSTARTTEXT_Y);
				isTextOn=1;
			}
		}

		r256(); // Randomize

		Input_Poll();
		HandleReset();
		HandleColorSwitch();

	} while(!isKeyPressed(KEY_START));

	Blit(&img_pressStart, bgColor, bgColor, PRESSSTARTTEXT_X, PRESSSTARTTEXT_Y);
	Blit(&img_gameover, bgColor, bgColor, GAMEOVERTEXT_X, GAMEOVERTEXT_Y);

	int i;
	for(i=0; i<MAX_MISSES; i++)
	{
		Sprite_SetCharacter(spMissStatFirst+i, &img_missEmpty);
	}

	score = 0;
	SetScoreDisplay(0);

	ResetPlayerElements();
}

void ResetField(void)
{
	ResetBatteryStat();
	ResetBattery();
	ResetEnemies();
	ResetRobotShots();
}

void ResetPlayerElements(void)
{
	ResetPlayer();
	ResetMover();
	ResetRobot();
	LCD_ON(spBatteryUpLast);
}

  // Hides the Player and Robot
void HidePlayerElements(void)
{
	int i;
	for(i=0; i<NUM_PLAYER_POS; i++)
	{
		LCD_OFF(spPlayerFirst + i);
	}

	for(i=0; i<NUM_PLAYER_COLS; i++)
	{
		Sprite_SetCharacter(spRobotFirst + robotPos, &img_robotTrack);
		LCD_OFF(spRobotHeadFirst + i);
	}
	Sprite_SetCharacter(spRobotFirst + robotPos, &img_robotStandEmpty);
	LCD_OFF(spBatteryUpLast);
}

void InitCheats(void)
{
	cheats.bGivesBatteries = B_GIVES_BATTERIES;
	cheats.invincible = INVINCIBLE;
	cheats.noBatteriesNeeded = NO_BATTERIES_NEEDED;
	cheats.robotAlwaysFollows = ROBOT_ALWAYS_FOLLOWS;
	cheats.shootAnywhere = SHOOT_ANYWHERE;
	cheats.unlimitedMisses = UNLIMITED_MISSES;
	cheats.catchBatteriesFromBack = CATCH_BATTERIES_FROM_BACK;
	cheats.bootIntoGreenMode = BOOT_INTO_GREEN_MODE;
	cheats.noGameMusic = NO_GAME_MUSIC;
	cheats.noIntroMusic = NO_INTRO_MUSIC;
	cheats.hardMode = HARD_MODE;
	cheats.easyMode = EASY_MODE;
	cheats.twoBatteriesPerCatch = TWO_BATTERIES_PER_CATCH;
	cheats.fullBatteries = FULL_BATTERIES;
}

void AgbMain(void)
{
	int shouldShoot;
	int shouldReleaseBattery;
	int playerSpriteShooting=-1;
	int playerShootingAnimCounter=0;

	int i;
	int gameBootCounter = 0;

	score=0;
	isGameRunning=0;
	screen = FrontBuffer;

	Sprite_ClearOAM(0, 128);

	r256init();
	InitCheats();
	InitSound();
	Input_Reset();

	REG_DISPCNT_L = SCREEN_MODE | BG2_ENABLE | OBJ_MAP_1D | OBJ_ENABLE;

	  // Play Intro
	DoIntro();

	REG_DISPCNT_L = SCREEN_MODE | BG2_ENABLE | OBJ_MAP_1D | OBJ_ENABLE;
	InitSound();

	  // Some Sprite Setup Stuff
	MapImagesToSprites();
	SetupDigitLookup();

	  // Set initial colorsceme
	if(cheats.bootIntoGreenMode)
	{
		fgColor=FG_COLOR_1;
		bgColor=BG_COLOR_1;
		bgfColor=BGF_COLOR_1;
	}
	else
	{
		fgColor=FG_COLOR_0;
		bgColor=BG_COLOR_0;
		bgfColor=BGF_COLOR_0;
	}

	// Blank screen to white-ish
	WaitForVSync();
	FillScreen(bgColor);

	  // Load Palette
	LoadSpritePalette8Bit(lcd_palette);

	  // Set Foreground Color
	SetFgColor(fgColor);

	  // Load Sprite Images
	LoadImages();

	  // Setup Sprites
	InitSprites();

	  // Boot Sequence
	if(!cheats.dontBoot)
	{
		for(gameBootCounter=0; gameBootCounter<50; gameBootCounter++)
		{
			WaitForVSync();
		}
		LCDTestDisplay();
		UpdateScreen();
		PlayBeep2();
		for(gameBootCounter=0; gameBootCounter<70; gameBootCounter++)
		{
			WaitForVSync();
		}
		  // Undo a few things that LCDTestDisplay() did
		LCD_OFF(spZap);
		LCD_OFF(spBattInc);
		for(i = 0; i < NUM_PLAYER_COLS*2; i++)
		{
			Sprite_SetCharacter(spPlayerFirst+i, &img_playerMiddle);
		}
		for(i = 0; i < NUM_ENEMY_COLS; i++)
		{
			Sprite_SetCharacter(spRobotFirst + i, &img_robot);
			LCD_OFF(spRobotHeadFirst + i);
		}
		Blit(&img_paused, bgColor, bgColor, PAUSEDTEXT_X, PAUSEDTEXT_Y);
		Blit(&img_pressStart, bgColor, bgColor, PRESSSTARTTEXT_X, PRESSSTARTTEXT_Y);
		Blit(&img_gameover, bgColor, bgColor, GAMEOVERTEXT_X, GAMEOVERTEXT_Y);
	}

	  // Setup Sprites Initial States
	ResetField();
	ResetPlayerElements();
	HidePlayerElements();
	SetScoreDisplay(0);
//	SetScoreDisplay(128-endOfSprites);  // OAM Sprites Available
//	SetScoreDisplay(lastTakenCharData-512);  // Char Mem Available
//	SetScoreDisplay(1024-lastTakenCharData);  // Char Mem Taken

	UpdateScreen();

	while(1)
	{
		if(songEnded)
		{
			StopMusic();
			songEnded = 0;
		}

		if(!isGameRunning)
		{
			DoGameNotRunning();
			isGameRunning = 1;

			if(shouldReset)
				return;

			StartMusic(2); //2=gameStartMelody
			while(!songEnded)
			{
				WaitForVSync();
			}
//			StopMusic();
//			songEnded = 0;

		}

		Input_Poll();

		  // Pause
		if(isKeyPressed(KEY_START))
		{
			DoPause();
			if(shouldReset)
				return;
		}

		HandleReset();
		if(shouldReset)
			return;
		HandleColorSwitch();

		// Update ---------
		if(isKeyPressed(KEY_B) && cheats.bGivesBatteries)
		{
			numBatteries++;
			DisplayBatteryStat(numBatteries);
		}

		  // Player
		shouldReleaseBattery=0;

		   // This is necessary to prevent the mover from glitching
		   // if the robotAlwaysFollows cheat is turned on
		   // when playerPosX != mover position
		if(cheats.robotAlwaysFollows && playerPosX != robotPos)
		{
			if(playerSpriteShooting != -1)
			{
				Sprite_SetCharacter(playerSpriteShooting, &img_playerMiddle);
				playerSpriteShooting = -1;
			}
			LCD_OFF(spPlayerFirst + playerPosX + (playerPosY * NUM_PLAYER_COLS));
			playerPosX = robotPos;  // This line is the main part of this fix
			LCD_ON(spPlayerFirst + playerPosX + (playerPosY * NUM_PLAYER_COLS));
		}

		if(isKeyPressed(KEY_LEFT))
		{
			if(playerPosX > 0)
			{
				  // If player was in shooting animation (or not) then
				  // just make sure these values are set back to defaults
				if(playerSpriteShooting != -1)
				{
					Sprite_SetCharacter(playerSpriteShooting, &img_playerMiddle);
					playerSpriteShooting = -1;
				}

				LCD_OFF(spPlayerFirst + playerPosX + (playerPosY * NUM_PLAYER_COLS));
				playerPosX--;
				LCD_ON(spPlayerFirst + playerPosX + (playerPosY * NUM_PLAYER_COLS));

				  // Mover and Robot
				if(playerPosY == NUM_PLAYER_ROWS-1 || cheats.robotAlwaysFollows)
				{
					robotPos--;
					SetMoverPos(robotPos);

					PlayLow1();
				}
				else
				{
					PlayTick5();
				}
			}
		}
		if(isKeyPressed(KEY_RIGHT))
		{
			  // Is player pushing the Battery Button?
			if(playerPosX == NUM_PLAYER_COLS-1 && playerPosY == 0 &&
				IS_LCD_ON(spBatteryUpLast))
			{
				shouldReleaseBattery = 1;
				Sprite_SetCharacter(spBatteryRelease, &img_battReleaseOpen);
				LCD_ON(spBatteryReleaseDoorOpen);
			}

			  // If player was in shooting animation (or not) then
			  // just make sure these values are set back to defaults
			if(playerSpriteShooting != -1)
			{
				Sprite_SetCharacter(playerSpriteShooting, &img_playerMiddle);
				playerSpriteShooting = -1;
			}

			if(playerPosX < NUM_PLAYER_COLS-1)
			{
				LCD_OFF(spPlayerFirst + playerPosX + (playerPosY * NUM_PLAYER_COLS));
				playerPosX++;
				LCD_ON(spPlayerFirst + playerPosX + (playerPosY * NUM_PLAYER_COLS));

				  // Mover and Robot
				if(playerPosY == NUM_PLAYER_ROWS-1 || cheats.robotAlwaysFollows)
				{
					robotPos++;
					SetMoverPos(robotPos);

					PlayLow1();
				}
				else
				{
					PlayTick5();
				}
			}
		}
		if(isKeyPressed(KEY_UP))
		{
			if(playerPosY > 0)
			{
				  // If player was in shooting animation (or not) then
				  // just make sure these values are set back to defaults
				if(playerSpriteShooting != -1)
				{
					Sprite_SetCharacter(playerSpriteShooting, &img_playerMiddle);
					playerSpriteShooting = -1;
				}

				LCD_OFF(spPlayerFirst + playerPosX + (playerPosY * NUM_PLAYER_COLS));
				playerPosY--;
				LCD_ON(spPlayerFirst + playerPosX + (playerPosY * NUM_PLAYER_COLS));

				PlayTick5();
			}
		}
		if(isKeyPressed(KEY_DOWN))
		{
			if(  (playerPosY < NUM_PLAYER_ROWS-2) || 
				((playerPosY == NUM_PLAYER_ROWS-2) && (playerPosX == robotPos)))
			{
				  // If player was in shooting animation (or not) then
				  // just make sure these values are set back to defaults
				if(playerSpriteShooting != -1)
				{
					Sprite_SetCharacter(playerSpriteShooting, &img_playerMiddle);
					playerSpriteShooting = -1;
				}

				LCD_OFF(spPlayerFirst + playerPosX + (playerPosY * NUM_PLAYER_COLS));
				playerPosY++;
				LCD_ON(spPlayerFirst + playerPosX + (playerPosY * NUM_PLAYER_COLS));

				PlayTick5();
			}
		}

		  // Shoot
		shouldShoot = 0;

		if(isKeyPressed(KEY_A) &&
			(numBatteries > 0 || cheats.noBatteriesNeeded) &&
			(playerPosY < 2 || cheats.shootAnywhere))
		{
			shouldShoot = 1;

			if(playerPosY < 2)  // This could be false if a cheat is on
			{
				playerSpriteShooting = spPlayerFirst + playerPosX + (playerPosY*NUM_PLAYER_COLS);
				Sprite_SetCharacter(playerSpriteShooting, &img_playerMiddleShoot);
				playerShootingAnimCounter = 0;
			}
		}

		if(playerSpriteShooting != -1)
		{
			playerShootingAnimCounter++;
			if(playerShootingAnimCounter >= PLAYERSHOOT_ANIM_DELAY)
			{
				Sprite_SetCharacter(playerSpriteShooting, &img_playerMiddle);
				playerSpriteShooting = -1;
			}
		}

		  // Misc gameplay objects
		DoRobotShots(shouldShoot, &numBatteries);
		DoEnemies();
		DoBattery(shouldReleaseBattery, &numBatteries, &score,
			playerPosX, playerPosY);

		if(shouldReset)
			return;

		// Draw ----------
		  // Flush everything to the screen
		UpdateScreen();
	}
}

//end of entry point


