// My Robot - Battery - battery.c
// Nick Sabalausky

#include "gba.h"
#include "userdefs.h"
#include "rand.h"

  // GBA Headers
#include "gfx.h"
#include "sprite.h"
#include "input.h"
#include "mathluts.h"

  // Game Headers
#include "gamedefs.h"
#include "intro.h"
#include "sound.h"
#include "blit.h"
#include "enemyCode.h"
#include "robotShotCode.h"
#include "battery.h"

extern img img_battReleaseClosed;

int battIncDelay;
int batteryFrameCount=0;
int shouldCreateBattery=0;

void ResetBattery(void)
{
	int i;
	for(i=spBatteryUpFirst; i < spBatteryUpLast; i++)
	{
		LCD_OFF(i);
	}
	for(i=spBatteryLeftFirst; i <= spBatteryLeftLast; i++)
	{
		LCD_OFF(i);
	}
	for(i=spBatteryFallFirst; i <= spBatteryFallLast; i++)
	{
		LCD_OFF(i);
	}
	Sprite_SetCharacter(spBatteryRelease, &img_battReleaseClosed);
	LCD_OFF(spBatteryReleaseDoorOpen);
	batteryFrameCount=0;
	shouldCreateBattery=0;
}

int IsBatteryUpClear(void)
{
	int i;

	for(i=spBatteryUpFirst; i <= spBatteryUpLast-1; i++)
	{
		if(IS_LCD_ON(i))
		{
			return 0;
		}
	}

	return 1;
}

void DoBattery(int shouldReleaseBattery, int *numBatteries, int* score,
			   u16 playerPosX, u16 playerPosY)
{
	if(shouldReleaseBattery && !shouldCreateBattery)
	{
		if(IsBatteryUpClear())
		{
			shouldCreateBattery = 1;
			PlayDown3();
		}
	}

	batteryFrameCount++;
	if(batteryFrameCount >= BATTERY_ANIM_DELAY)
	{
		batteryFrameCount = 0;

		if(IS_LCD_ON(spBattInc))
		{
			battIncDelay++;
			if(battIncDelay==3)
			{
				battIncDelay=0;
				LCD_OFF(spBattInc);
			}
		}

		DoBatteryUpdate(numBatteries, score, playerPosX, playerPosY);

		if(shouldCreateBattery)
		{
			shouldCreateBattery = 0;
			LCD_OFF(spBatteryUpLast);
			LCD_ON(spBatteryUpLast-1);
		}
	}
}

extern void DisplayBatteryStat(int numBatteries);
extern void SetScoreDisplay(int score);
void DoBatteryUpdate( int* numBatteries, int* score, u16 playerPosX, u16 playerPosY)
{
	int i;
	static u16 batteryLastRow=0;
	static u16 batteryLeftToFallFrom=0;
//	static int batteryFallAnimDelay=0;


//	batteryFallAnimDelay++;
//	if(batteryFallAnimDelay >= 2)
//	{
//		batteryFallAnimDelay=0;

		  // Battery Fall Last Row
		batteryLastRow = spBatteryFallFirst+(NUM_BATTFALL-NUM_BATTFALL_COLS);
		for(i = NUM_BATTFALL_COLS-1; i >= 0; i--)
		{
			if(IS_LCD_ON(batteryLastRow+i))
			{
				LCD_OFF(batteryLastRow+i);

				  // Check if the battery has been caught
				if(playerPosX == i &&
					(playerPosY == 0 || cheats.catchBatteriesFromBack))
				{
					if(*numBatteries < 0) // Possible with a cheat
						*numBatteries = 0;
	
					if(cheats.twoBatteriesPerCatch)
						*numBatteries += 2;
					else
						*numBatteries += NUM_BATTERIES_PER_CATCH;

					if(*numBatteries > 8)
						*numBatteries = 8;

					DisplayBatteryStat(*numBatteries);
					
					*score += SCORE_PER_BATTERY;
					SetScoreDisplay(*score);

					LCD_ON(spBattInc);
					battIncDelay=0;
					PlayUp2();
				}
			}
		}

		  // Battery Fall
		for(i=batteryLastRow-1; i >= spBatteryFallFirst; i--)
		{
			if(IS_LCD_ON(i))
			{
				LCD_OFF(i);
				LCD_ON(i+NUM_BATTFALL_COLS);
			}
		}
//	}

	  // Battery Left
	for(i = spBatteryLeftFirst; i <= spBatteryLeftLast; i++)
	{
		if(IS_LCD_ON(i))
		{
			LCD_OFF(i);

			if(i > spBatteryLeftFirst + batteryLeftToFallFrom)
				LCD_ON(i-1);
			else
			{
				LCD_ON(spBatteryFallFirst + batteryLeftToFallFrom/2);
			}
		}
	}


//	if(batteryFallAnimDelay == 0) // Zero because it's wrapped
//	{
	  // Battery Up
	for(i = spBatteryUpFirst; i <= spBatteryUpLast-1; i++)
	{
		if(IS_LCD_ON(i))
		{
			LCD_OFF(i);

			if(i > spBatteryUpFirst)
				LCD_ON(i-1);
			else
			{
				LCD_ON(spBatteryLeftLast);

				  // Choose column (batteryLeft LCD) where battery
				  // will start to fall.
				batteryLeftToFallFrom = ((r256() & 0xC000) >> 14) * 2;

				  // Turn On Next Battery Waiting for Release
				LCD_ON(spBatteryUpLast);
			}

			if(i == spBatteryUpLast-3)
			{
				  // Reset Battery Release
				Sprite_SetCharacter(spBatteryRelease, &img_battReleaseClosed);
				LCD_OFF(spBatteryReleaseDoorOpen);
			}
		}
	}
//	}
/*	for(i = spRobotShotFirst; i < spRobotShotFirst+NUM_ENEMY_COLS; i++)
	{
		LCD_OFF(i);
	}

	for(; i <= spRobotShotLast; i++)
	{
		  // If enemy exists at this location
		if(IS_LCD_ON(i))
		{
			LCD_OFF(i);
			LCD_ON(i-NUM_ENEMY_COLS);
		}
	}
	*/
}

//extern void SetScoreDisplay(int score);
/*void DoRobotBatteryCollide(void)
{
	int i;
	  // NUM_ENEMIES is also the number of robot shots
	for(i=0; i < NUM_ENEMIES; i++)
	{
		  // If both the enemy and robot shot in position 'i'
		  // are turned on, then they've collided.
		if( IS_LCD_ON(spEnemyFirst + i) &&
			IS_LCD_ON(spRobotShotFirst + i) )
		{
			PlayDown2();

			LCD_OFF(spEnemyFirst + i);
			LCD_OFF(spRobotShotFirst + i);

			score += 4;
			SetScoreDisplay(score);
		}
	}
}
*/
// END OF FILE //////////////////////