// My Robot - Enemy Code
// Nick Sabalausky

#include "gba.h"
#include "userdefs.h"
#include "rand.h"

  // GBA Headers
#include "gfx.h"
#include "sprite.h"
#include "input.h"

  // Game Headers
#include "gamedefs.h"
#include "sound.h"
#include "enemyCode.h"
#include "robotShotCode.h"

int enemyFrameCount=0;
int shouldCreate=0;
int animTicksWithoutCreate=0;

void ResetEnemies(void)
{
	int i;
	for(i=spEnemyFirst; i <= spEnemyLast; i++)
	{
		LCD_OFF(i);
	}
	enemyFrameCount=0;
	shouldCreate=0;
	animTicksWithoutCreate=0;
}

void DoEnemies(void)
{
	int animTicksWoCreateThreshold=MAX_ANIM_TICKS_WO_CREATE;
	if(cheats.hardMode)
		animTicksWoCreateThreshold=1;
	if(cheats.easyMode)  // Easy Mode overrules hard mode (if both are set)
		animTicksWoCreateThreshold=MAX_ANIM_TICKS_WO_CREATE;

	enemyFrameCount++;
	if(enemyFrameCount >= ENEMY_ANIM_DELAY)
	{
		enemyFrameCount = 0;

		if(DoEnemyUpdate()==1)
		{
			//if(isKeyPressed(KEY_B))
			if(r256() < 0xFFFF/2)
			{
				if(!cheats.easyMode || animTicksWithoutCreate >= 1)
				{
					shouldCreate = 1;
					animTicksWithoutCreate=0;
				}
			}
			else
			{
				animTicksWithoutCreate++;
				if(animTicksWithoutCreate >= animTicksWoCreateThreshold)
				{
					shouldCreate = 1;
					animTicksWithoutCreate=0;
				}
			}
		}

		if(shouldCreate)
		{
			shouldCreate = 0;
			DoEnemyCreation();
		}
	}
}

void DoEnemyCreation(void)
{
	u16 col;
	enumSprites spriteIndex;

	col = (r256() & 0xC000) >> 14;
	spriteIndex = spEnemyFirst + col;

	LCD_ON(spriteIndex);

	PlayTick2();
}

extern void LoseTry(u16 guiltySprite);
  // Returns 0 if enemy got to bottom and player lost a try
  // Returns 1 otherwise
int DoEnemyUpdate(void)
{
	int i;
	int shouldMakeSound=0;

	  // Bottom Row
	for(i = spEnemyLast; i > spEnemyLast-NUM_ENEMY_COLS; i--)
	{
		  // If enemy exists at this location
		if(IS_LCD_ON(i))
		{
			LCD_OFF(i);

			if(!cheats.invincible)
			{
				LoseTry(i);
				return 0;
			}
		}
	}

	for(; i >= spEnemyFirst; i--)
	{
		  // If enemy exists at this location
		if(IS_LCD_ON(i))
		{
			LCD_OFF(i);
			LCD_ON(i+NUM_ENEMY_COLS);
			shouldMakeSound = 1;
		}
	}

	if(shouldMakeSound)
		PlayTick2();

	return 1;
}

// END OF FILE ///////////////////
