// My Robot - gamedefs.h
// Nick Sabalausky

#ifndef _GAMEDEFS_H_
#define _GAMEDEFS_H_

typedef volatile unsigned char    vu8;
typedef volatile unsigned short   vu16;
typedef volatile unsigned long    vu32;

typedef volatile signed char    vs8;
typedef volatile signed short   vs16;
typedef volatile signed long    vs32;

  // Colors
#define BG_COLOR_0	RGB(29, 30, 31)
#define BGF_COLOR_0	RGB(15, 13, 11)
#define FG_COLOR_0	0x0000

#define BG_COLOR_1	0x0000
#define BGF_COLOR_1	RGB(3, 19, 6)
#define FG_COLOR_1	RGB(0, 30, 3)

#define FG_COLOR_INDEX 0x0F

//// Cheats //////
// 0:off   1:on //
#define B_GIVES_BATTERIES		0
#define NO_BATTERIES_NEEDED		0
#define SHOOT_ANYWHERE			0
#define ROBOT_ALWAYS_FOLLOWS	0
#define INVINCIBLE				0
#define UNLIMITED_MISSES			0
#define CATCH_BATTERIES_FROM_BACK	0
#define BOOT_INTO_GREEN_MODE	0
#define DONT_BOOT				0
#define NO_GAME_MUSIC			0
#define NO_INTRO_MUSIC			0
#define HARD_MODE				0
#define EASY_MODE				0
#define TWO_BATTERIES_PER_CATCH		0
#define FULL_BATTERIES				0

typedef struct t_cheats
{
	u16 bGivesBatteries;
	u16 noBatteriesNeeded;
	u16 shootAnywhere;
	u16 robotAlwaysFollows;
	u16 invincible;
	u16 unlimitedMisses;
	u16 catchBatteriesFromBack;
	u16 bootIntoGreenMode;
	u16 dontBoot;
	u16 noGameMusic;
	u16 noIntroMusic;
	u16 hardMode;
	u16 easyMode;
	u16 twoBatteriesPerCatch;
	u16 fullBatteries;
} cheatTable;

extern cheatTable cheats;

//// Gameplay //////
#define SCORE_PER_BATTERY	4
#define SCORE_PER_ENEMYHIT	8

#define BATTERIES_INITIAL 2
#define NUM_BATTERIES_PER_CATCH 1
#define MAX_BATTERIES	8

//// Animation ////
#define ENEMY_ANIM_DELAY		100
#define ROBOTSHOT_ANIM_DELAY	30
#define PRESSSTART_ANIM_DELAY	20
#define PAUSE_ANIM_DELAY		20
#define BATTERY_ANIM_DELAY		10
#define PLAYERSHOOT_ANIM_DELAY	30

#define MAX_ANIM_TICKS_WO_CREATE  4

#define LOSETRY_NUM_FLASHES  6
#define LOSETRY_ANIM_DELAY  35

//// Position and Movement ////

  // Enemy
#define ENEMY_START_X    137
#define ENEMY_START_Y    2
#define ENEMY_SPACING_X  25
#define ENEMY_SPACING_Y  18

#define ENEMY_STEP_Y  18

#define NUM_ENEMY_COLS    4
#define NUM_ENEMY_ROWS    5
#define NUM_ENEMIES  (NUM_ENEMY_COLS * NUM_ENEMY_ROWS)

  // Robot
#define ROBOT_START_X	135
#define ROBOT_Y			102
#define ROBOTHEAD_OFFSET_X	7
#define ROBOTHEAD_OFFSET_Y	(-5)

#define ROBOT_STEP_X	25
#define ROBOT_MIN_X		137
#define ROBOT_MAX_X		(ROBOT_MIN_X + (ROBOT_STEP_X*(NUM_ENEMY_COLS-1)))

#define ROBOTSHOT_OFFSET_X   4
#define ROBOTSHOT_OFFSET_Y   1

#define ROBOT_INITIAL_POS	1

  // Mover
#define MOVER_X			0
#define MOVER_Y			114
#define MOVER_SPACING_X	26
#define NUM_MOVER_SECS	5
#define NUM_MOVER_LEFT	(NUM_MOVER_SECS - 2)

  // Player
#define PLAYER_STEP_X	26
#define PLAYER_STEP_Y	20

#define NUM_PLAYER_ROWS		3
#define NUM_PLAYER_COLS		NUM_ENEMY_COLS
#define NUM_PLAYER_POS		(NUM_PLAYER_COLS * NUM_PLAYER_ROWS)

#define PLAYER_X	MOVER_X
#define PLAYER_Y	(MOVER_Y - (PLAYER_STEP_Y * (NUM_PLAYER_ROWS-1)))

#define PLAYER_INIT_POSX	1
#define PLAYER_INIT_POSY	1

  // Score
#define NUM_SCORE_DIGITS  5
#define DIGIT_WIDTH  11

     // These specify the position of the least significant digit
#define SCORE_X		123
#define SCORE_Y		139

  // Batteries
#define BATTERYSTAT_X	136
#define BATTERYSTAT_Y	143
#define BATTERYSTAT_SPACING_X	16
#define BATTERYSTAT_SPACING_Y	8

#define NUM_BATTFALL_ROWS	5
#define NUM_BATTFALL_COLS	4
#define NUM_BATTFALL		(NUM_BATTFALL_ROWS*NUM_BATTFALL_COLS)
#define NUM_BATTUP			8
#define NUM_BATTLEFT		8

#define BATTFALL_START_X	5
#define BATTFALL_START_Y	20
#define BATTFALL_SPACING_X	26
#define BATTFALL_SPACING_Y	10

#define BATTUP_X			(BATTRELEASE_X - 1)
#define BATTUP_START_Y		(-3)
#define BATTUP_SPACING_Y	10
#define BATTUP_LAST_Y		(BATTRELEASE_Y - 3)

#define BATTLEFT_START_X	10
#define BATTLEFT_Y			1
#define BATTLEFT_SPACING_X	13

#define BATTRELEASE_X		99
#define BATTRELEASE_Y		74

#define BATTRELEASEDOOROPEN_X	108
#define BATTRELEASEDOOROPEN_Y	52

  // Text
#define BATTERYTEXT_X	136
#define BATTERYTEXT_Y	137

#define MISSTEXT_X	201
#define MISSTEXT_Y	137

#define SCORETEXT_X	59
#define SCORETEXT_Y	137

#define TEXT_HEIGHT	7

#define GAMEOVERTEXT_X		MISSTEXT_X
#define GAMEOVERTEXT_Y		(MISSTEXT_Y + (TEXT_HEIGHT*2) + 1)

#define PAUSEDTEXT_X		2
#define PAUSEDTEXT_Y		(MYROBOTTEXT_Y + (TEXT_HEIGHT * 2) + 2)/*137*/

#define PRESSSTARTTEXT_X	PAUSEDTEXT_X
#define PRESSSTARTTEXT_Y	(MYROBOTTEXT_Y + TEXT_HEIGHT + 2)

#define MYROBOTTEXT_X		PAUSEDTEXT_X
#define MYROBOTTEXT_Y		137/*(PAUSEDTEXT_Y + (TEXT_HEIGHT * 2))*/

  // ConsoleFever
#define CONSOLEFEVER_GAME_X		(240-90)
#define CONSOLEFEVER_GAME_Y		1
#define CONSOLEFEVER_INTRO_X	0
#define CONSOLEFEVER_INTRO_Y	(100/2)
//#define CONSOLEFEVER_INTRO_X	129
//#define CONSOLEFEVER_INTRO_Y	0
//#define CONSOLEFEVER_INTRO_Y	91
//#define CONSOLEFEVER_INTRO_Y	151

  // Press Start To Skip This Intro
#define PRESSSTARTINTRO_X	0
#define PRESSSTARTINTRO_Y	153

  // Miss Status
#define MAX_MISSES	3
#define MISS_X		MISSTEXT_X
#define MISS_Y		(MISSTEXT_Y + TEXT_HEIGHT - 1)
#define MISS_SPACING_X	13

#define ZAP_X	108
#define ZAP_Y	88

#define BATTINC_X	110
#define BATTINC_Y	118

	// Macros

// These take in a sprite (ie. an enumSprite)
#define LCD_ON(a)		Sprite_SizeDoubleOff((a))
#define LCD_OFF(a)		Sprite_SizeDoubleOn((a))
#define IS_LCD_ON(a)	(!Sprite_IsSizeDoubleOn((a)))
#define LCD_TOGGLE(a)	Sprite_SizeDoubleToggle((a))

  // Sprites
typedef enum t_enumSprites
{
	spPlayerFirst=0,
	spPlayerLast = spPlayerFirst + NUM_PLAYER_POS-1,
	
	spEnemyFirst,
	spEnemyLast = spEnemyFirst + NUM_ENEMIES-1,
	
	spRobotFirst,
	spRobotLast = spRobotFirst + NUM_ENEMY_COLS-1,
	spRobotHeadFirst,
	spRobotHeadLast = spRobotHeadFirst + NUM_ENEMY_COLS-1,
	spRobotShotFirst,
	spRobotShotLast = spRobotShotFirst + NUM_ENEMIES-1,
	
	spScoreFirst, // First is the least significant digit
	spScoreLast = spScoreFirst + NUM_SCORE_DIGITS-1,
	
	spMoverSecFirst,
	spMoverSecLast = spMoverSecFirst + NUM_MOVER_SECS-1,
	spMoverLeftFirst,
	spMoverLeftLast = spMoverLeftFirst + NUM_MOVER_LEFT-1,
	spMoverRightFirst = spMoverLeftLast,
	spMoverRightLast = spMoverRightFirst + NUM_MOVER_SECS-1,

	spBatteryStatFirst,
	spBatteryStatLast = spBatteryStatFirst + MAX_BATTERIES-1,

	spMissStatFirst,
	spMissStatLast = spMissStatFirst+MAX_MISSES-1,

	spBatteryUpFirst,
	spBatteryUpLast = spBatteryUpFirst + NUM_BATTUP-1,
	spBatteryLeftFirst,
	spBatteryLeftLast = spBatteryLeftFirst + NUM_BATTLEFT-1,
	spBatteryFallFirst,
	spBatteryFallLast = spBatteryFallFirst + NUM_BATTFALL-1,

	spBatteryRelease,
	spBatteryReleaseDoorOpen,
	spZap,
	spBattInc,

	endOfSprites
} enumSprites;

  // Digit Lookup
extern img* digitImg[10];

extern vu8 melody;
extern vu8 songEnded;
extern vu16 songIndex;
extern int score;

#endif

// END OF FILE ////////////////////////////