// GBA Graphics System - gfx.c
// Nick Sabalausky

#include "gba.h"
#include "userdefs.h"
#include "gfx.h"

u16 *screen;

void inline WaitForVSync()
{
	while(*SCANLNE_COUNT >= SCREEN_HEIGHT) {/*Do nothing*/}
	while(*SCANLNE_COUNT < SCREEN_HEIGHT) {/*Do nothing*/}
}

void inline Flip()
{
	  //  REG_DISPCNT_L, but non-volitile
	if(REG_DISPCNT_L & BACKBUFFER)
	{
		FLIP_TO_BUFFER_2;
	}
	else
	{
		FLIP_TO_BUFFER_1;
	}
}

typedef struct t_u32byte
{ 
    u32 word1; 
    u32 word2; 
    u32 word3; 
    u32 word4; 
} u32byte; 

  // Apperently, the GBA's CPU can copy 32 bytes at a time.
  // Thanks to Miked0801 on the gbadev.org forums for pointing this out.
void FillScreen(u16 color)
{
	u32 fillVal = color;
	fillVal = fillVal | (fillVal << 16);
	u32byte fillData = {fillVal, fillVal, fillVal, fillVal};

	u32byte* screenPtr;
	for(screenPtr=(u32byte*)screen; (u32)screenPtr<(u32)screen + SCREEN_WIDTH*SCREEN_HEIGHT*(SCREEN_BPP/8); screenPtr++)
	{
		*screenPtr = fillData;
	}
}

  // This is slow and may not work in mode 4
  // All coordinates are inclusive
void DrawHLineToFrom(u16 color, u16 y, u16 x1, u16 x2)
{
	int i;
	int temp;

	  // Make sure x1 <= x2
	if(x1 > x2)
	{
		temp = x1;
		x1 = x2;
		x2 = temp;
	}

	for(i = x1; i <= x2; i++)
	{
		PLOT_PIXEL(i, y, color);
	}
}

  // This is slow and may not work in mode 4
  // All coordinates are inclusive
void DrawHLineLength(u16 color, u16 y, u16 x1, u16 length)
{
	int i;
	int x2 = x1 + length;

	for(i = x1; i < x2; i++)
	{
		PLOT_PIXEL(i, y, color);
	}
}

  // This is slow and may not work in mode 4
  // All coordinates are inclusive
void DrawVLineToFrom(u16 color, u16 x, u16 y1, u16 y2)
{
	int i;
	u16 temp;

	  // Make sure y1 <= y2
	if(y1 > y2)
	{
		temp = y1;
		y1 = y2;
		y2 = temp;
	}

	for(i = y1; i <= y2; i++)
	{
		PLOT_PIXEL(x, i, color);
	}
}

  // This is slow and may not work in mode 4
  // All coordinates are inclusive
void DrawVLineLength(u16 color, u16 x, u16 y1, u16 length)
{
	int i;
	u16 y2 = y1 + length;

	for(i = y1; i < y2; i++)
	{
		PLOT_PIXEL(x, i, color);
	}
}

  // This is slow and may not work in mode 4
  // All coordinates are inclusive
void DrawRectToFrom(u16 color, u16 x1, u16 y1, u16 x2, u16 y2)
{
	DrawHLineToFrom(color, y1, x1, x2);
	DrawHLineToFrom(color, y2, x1, x2);
	DrawVLineToFrom(color, x1, y1, y2);
	DrawVLineToFrom(color, x2, y1, y2);
}

  // This is slow and may not work in mode 4
  // All coordinates are inclusive
void DrawRectSize(u16 color, u16 x1, u16 y1, u16 width, u16 height)
{
	DrawHLineLength(color, y1,        x1, width);
	DrawHLineLength(color, y1+height, x1, width);
	DrawVLineLength(color, x1,        y1, height);
	DrawVLineLength(color, x1+width,  y1, height);
}

// END OF FILE //////////////////////////
