// GBA Graphics System - gfx.h
// Nick Sabalausky

#ifndef _GFX_H_
#define _GFX_H_

#define MODE_0          0x00
#define MODE_1          0x01
#define MODE_2          0x02
#define MODE_3          0x03
#define MODE_4          0x04
#define MODE_5          0x05

#define MODE_3_WIDTH	240
#define MODE_3_HEIGHT	160
#define MODE_3_BPP		16

#define MODE_4_WIDTH	240
#define MODE_4_HEIGHT	160
#define MODE_4_BPP		8

#define MODE_5_WIDTH	160
#define MODE_5_HEIGHT	128
#define MODE_5_BPP		16

  // Special Addresses
#define SCANLNE_COUNT  (volatile u16*)0x04000006

  // Structs
// struct img is tied closely to the gbaimg tool
typedef struct img_t
{
	u16 width;
	u16 height;
	u16 shape;
	u16 size;

	const u16 *pal;
	const u16 *data;

	s16 charMemIndex;
} img;

  // MACROS
#define PLOT_PIXEL(x, y, c)  screen[(x) + ((y) * SCREEN_WIDTH)] = c

#define FLIP_TO_BUFFER_1  REG_DISPCNT_L |=  BACKBUFFER; screen = FrontBuffer
#define FLIP_TO_BUFFER_2  REG_DISPCNT_L &= ~BACKBUFFER; screen = BackBuffer

#define GET_RED(c)    (((c) & (0x001F      ))      )
#define GET_GREEN(c)  (((c) & (0x001F << 5 )) >> 5 )
#define GET_BLUE(c)   (((c) & (0x001F << 10)) >> 10)

// DRAW_SCREEN(a)
u32 _draw_screen_index;
#define DRAW_SCREEN(a)     \
for(_draw_screen_index=0; _draw_screen_index<(SCREEN_WIDTH * SCREEN_HEIGHT *(SCREEN_BPP/8))/2; _draw_screen_index++) \
{  \
		screen[_draw_screen_index] = a[_draw_screen_index];  \
}  _draw_screen_index=0

// Don't use this fillscreen, it's horribly slow.
// Use FillScreen() below, it's much faster.
// FILL_SCREEN(a)
u32 _fill_screen_index;
#define FILL_SCREEN(a)     \
for(_fill_screen_index=0; _fill_screen_index<(SCREEN_WIDTH * SCREEN_HEIGHT *(SCREEN_BPP/8))/2; _fill_screen_index++) \
{  \
		screen[_fill_screen_index] = (a);  \
}  _fill_screen_index=0

// LOAD_BGPALETTE(a)
u32 _load_bgpalette_index;
#define LOAD_BGPALETTE(a)     \
for(_load_bgpalette_index=0; _load_bgpalette_index<256; _load_bgpalette_index++) \
{  \
		BGPaletteMem[_load_bgpalette_index] = a[_load_bgpalette_index];  \
}  _load_bgpalette_index=0

  // Globals
extern u16* screen;

  // Functions
void inline WaitForVSync();
void inline Flip();

void FillScreen(u16 color);

  // These are slow and may not work in mode 4
  // All coordinates are inclusive
void DrawHLineToFrom(u16 color, u16 y, u16 x1, u16 x2);
void DrawHLineLength(u16 color, u16 y, u16 x1, u16 length);
void DrawVLineToFrom(u16 color, u16 x, u16 y1, u16 y2);
void DrawVLineLength(u16 color, u16 x, u16 y1, u16 length);
void DrawRectToFrom(u16 color, u16 x1, u16 y1, u16 x2, u16 y2);
void DrawRectSize(u16 color, u16 x1, u16 y1, u16 width, u16 height);

#endif
// END OF FILE //////////////////////////
