// Taken from Pern Project.
// I wrote the HBLANK function and made a few other minor changes.
// Nick Sabalausky

#define INTERUPT_C

#include "gba.h"
#include "gfx.h"
#include "gamedefs.h"
#include "sound.h"
#include "intro.h"
//#include "headers/dma.h"
//#include "headers/sprite.h"
//#include "headers/interupts.h"

extern fp intr_main;
//these are my function definitios to let the c compiler know what I am talking about

void VBLANK(void) __attribute__ ((section(".iwram")));
void HBLANK(void) __attribute__ ((section(".iwram")));
void DMA0(void);
void DMA1(void);
void DMA2(void);
void DMA3(void);
void VCOUNT(void);
void TIMER0(void);
void TIMER1(void);
void TIMER2(void);
void TIMER3(void);
void KEYBOARD(void);
void CART(void);
void COMUNICATION(void);


//here is a function to enable interupts.  If it is VBLANK or HBLANK it sets REG_DISPSTAT to the apropriate values
//all others you will have to do on your own.

void EnableInterupts(u16 interupts)
{
	REG_IME = 0;  //probably not necessary but not a good idea to change interupt registers when an interupt is ocuring


	if(interupts == INT_VBLANK)
		REG_DISPSTAT |= 0x8;

	if(interupts == INT_HBLANK)
		REG_DISPSTAT |= 0x10;
	

	REG_IE |= interupts;

	REG_IME = 1;

}

void DisableInterupts(u16 interupts)
{
	REG_IE &= ~interupts;

//	if(interupts == INT_ALL) //this is were that ALL comes in
//		REG_IME = 0;  //disable all interupts;
}




//here is our table of function pointers.  I added the definition of fp to gba.h because I figure it would be usefull
//to have elsewhere
//typedef void (*fp)(void);   this is the definition you will find in gba.h.  It is just delaring fp to represent a pointer
//to a function




//all our interupt functions empty for now.

void VBLANK(void)
{	
	REG_IF |= INT_VBLANK;
		
}

u32 shiftX[16] __attribute__ ((section(".iwram"))) ;
u32 shiftX[16] = {   0x0000100,  0x00001E0,  0x0000280,  0x0000300,
					 0x0000280,  0x00001E0,  0x0000100,  0x0000000, 
					-0x0000100, -0x00001E0, -0x0000280, -0x0000300,
					-0x0000280, -0x00001E0, -0x0000100,  0x0000000  }; 

void HBLANK(void)
{
	static u16 waveOffset=0;
	u16 vcount = REG_VCOUNT;
	u16 mosaicOffEnd;

	  // Turn waves on/off
	if(vcount == 99)
	{
		REG_BG2PD = -0x0080;
		REG_BG2Y  = ((vcount>>1)<<8);
	}
	else if(vcount == 161)
	{
		REG_BG2PA = 0x0080;

		REG_BG2PD = 0x0080;
		REG_BG2Y  = 0;
		REG_BG2X  = 0;

		waveOffset--;
		waveOffset &= 0x000F;
	}

	  // Turn off mosaic over URL display
	mosaicOffEnd = 100+(100-(consoleFeverY>>3))-1;
	if(vcount >= ((consoleFeverY>>3)-1) && vcount < mosaicOffEnd)
	{
		REG_BG2CNT &= ~0x0040;
	}
	else if (vcount >= mosaicOffEnd)
	{
		REG_BG2CNT |= 0x0040;
	}

	  // Waves
	if(vcount >= 100 && vcount < 160)
	{
		REG_BG2X = shiftX[(vcount+waveOffset) & 0x000F] + ((vcount-100)<<8) + 0x0310;
		REG_BG2PA = 0x007A - ((vcount-100)<<1);
	}

	REG_IF |= INT_HBLANK;
}
void DMA0(void)
{
	
	REG_IF |= INT_DMA0;

}
void DMA1(void)
{
	
	REG_IF |= INT_DMA1;

}
void DMA2(void)
{
	
	REG_IF |= INT_DMA2;

}
void DMA3(void)
{
	REG_IF |= INT_DMA3;

}
void VCOUNT(void)
{
	REG_IF |= INT_VCOUNT;

}

vu8 songEnded=0;
vu16 songIndex=0;
vu8 melody;  // 0: introMelody
			 // 1: gameoverMelody
			 // 2: gamestartMelody

#define DIFF 5
u8 introMelody[32] = {
5+DIFF, 5+DIFF, 13+DIFF, 5+DIFF, 14+DIFF-0, 13+DIFF, 5+DIFF, 8+DIFF, 5+DIFF, 5+DIFF, 13+DIFF, 5+DIFF, 14+DIFF-0, 13+DIFF, 5+DIFF, 8+DIFF, 
5, 5, 13, 5, 14, 13, 5, 8, 5, 5, 13, 5, 14, 13, 5, 8
};

#define DIFF2 5
u8 gameOverMelody[11] = {
14+DIFF2,14+DIFF2,8+DIFF2,8+DIFF2,11+DIFF2,11+DIFF2,6+DIFF2,6+DIFF2,3+DIFF2,3+DIFF2,3+DIFF2
};

u8 gameStartMelody[6] = {
//12+DIFF2,10+DIFF2,6+DIFF2,14+DIFF2,12+DIFF2,10+DIFF2,18+DIFF2,18+DIFF2,18+DIFF2
15, 15, 17, 15, 23, 23
};

u16 introPercussion[32] = {
0, 0, 0x8063, 0, 0, 0x8063, 0, 0, 0, 0, 0x8063, 0, 0, 0x8063, 0, 0,
0, 0, 0x8063, 0, 0, 0x8063, 0x8062, 0x8062, 0, 0, 0x8063, 0, 0, 0x8063, 0x8062, 0x8062
};

u16 introEffect[32] = {
2, 0, 2, 2, 2, 0,   0,   0, 2, 0, 2, 2, 2, 0,   0,   0,
2, 0, 2, 2, 0, 0, 1|4, 1|4, 2, 0, 2, 0, 0, 4, 1|4, 1|4
};

extern vu8 enemyChangeDirFlag;
extern vu8 enemyFangFlag;

void TIMER0(void)
{
	if(songEnded)
	{
		REG_IF |= INT_TIMER0;
		return;
	}

	u16 perc;
	perc = introPercussion[songIndex];
	if(melody == 0 && introPercussion[songIndex] != 0)
	{
		REG_SOUND4CNT_L = 0xF112;
		REG_SOUND4CNT_H = perc;

	}

	if(melody == 0)  // introMelody only
	{
		mosaicFlag = introEffect[songIndex] & 0x01;
		enemyChangeDirFlag = (introEffect[songIndex] & 0x02)>>1;
		enemyFangFlag = introEffect[songIndex]>>2;
	}

	switch(melody)
	{
	case 0:
		REG_SOUND3CNT_H = 0x2080;
		REG_SOUND3CNT_X = 0xC000 | freq[introMelody[songIndex]];
		songIndex++;
		if(songIndex==32)
			songIndex=0;
		break;
	case 1:
		  // Delay so the first note doesn't get cut off
		if(songIndex==0)
		{
			songIndex++;
			break;
		}
		
		REG_SOUND3CNT_H = 0x2080;
		REG_SOUND3CNT_X = 0xC000 | freq[gameOverMelody[songIndex-1]];
		songIndex++;
		if(songIndex==11+1)
			songEnded=1;
		break;
	case 2:
		  // Delay so the first note doesn't get cut off
		if(songIndex==0)
		{
			songIndex++;
			break;
		}
		
		REG_SOUND3CNT_H = 0x2080;
		REG_SOUND3CNT_X = 0xC000 | freq[gameStartMelody[songIndex-1]];
		songIndex++;
		if(songIndex==6+1)
			songEnded=1;
		break;
	}

	REG_IF |= INT_TIMER0;
}
void TIMER1(void)
{
	REG_IF |= INT_TIMER1;

}
void TIMER2(void)
{
	REG_IF |= INT_TIMER2;

}
void TIMER3(void)
{
	REG_IF |= INT_TIMER3;
}
void KEYBOARD(void)
{
	REG_IF |= INT_TIMER3;

}
void CART(void)
{
	REG_IF |= INT_CART;

}
void COMUNICATION(void)
{
	REG_IF |= INT_COMM;

}

//here is our table of functions.  Once interupts are enabled in crt0.s you will not be able to link your code if this table is not pressent.
//the interupts must be listed in that order.

fp IntrTable[]  = 
{
	VBLANK,
	HBLANK,
	VCOUNT,
	TIMER0,
	TIMER1,
	TIMER2,
	TIMER3,
	COMUNICATION,
	DMA0,
	DMA1,
	DMA2,
	DMA3,
	KEYBOARD,
	CART
};

