// My Robot - Enemy Code
// Nick Sabalausky

#include "gba.h"
#include "userdefs.h"
#include "rand.h"

  // GBA Headers
#include "gfx.h"
#include "sprite.h"
#include "input.h"
#include "mathluts.h"

  // Game Headers
#include "gamedefs.h"
#include "intro.h"
#include "sound.h"
#include "blit.h"
#include "enemyCode.h"
#include "robotShotCode.h"

  // Images
#include "imagesIntro.h"
#include "ImagesFont.h"
#include "ImagesFontSymbol.h"

u16 colorRotOn;

u16 colorLookup[2];
u16 colorR[2];
s16 colorRInc[2];
u16 colorG[2];
s16 colorGInc[2];
u16 colorB[2];
s16 colorBInc[2];
u16 colorDelay;

typedef enum t_introSprites
{
	spEnemy1=16,
	spEnemy2,
	spSkipIntroL,
	spSkipIntroR,
	endOfIntroSprites
} introSprites;

extern img img_consolefever;
extern img img_tpgaming;

img* blitURL = &img_consolefever;
int consoleFeverDelay=0;
int consoleFeverState=0;  // 0=Up 1=Show 2=Down 3=Wait
vs16 consoleFeverY = CONSOLEFEVER_INTRO_Y << 4;

int lettersYDegree[16];

vu16 mosaicFlag;

const char scrollyText[] = 
"Heya, this is Nitrode (aka Abscissa) from Twisted Pair Gaming (formerly Abscissa Software). This is \"My Robot\", my entry for the PDRoms competition 2.5. I hope you like it! If you are running this on the VisualBoy Advance emulator, you can activate a few simple cheat codes by clicking on the \"Cheat\" menu and loading the MyRobot.clt file.  I am also making the source available in case anyone wants to see it, but be warned: it's messy (A two-week development cycle can do that ;) ).  So have fun with the source - but don't steal anything without giving credit, because that would really irk me and I would have to resort to telling yo mamma jokes - and we don't want that, do we?     Sooo....what next?    I feel like plugging cool stuff...just because I can.  First off, visit consolefever.com, those people rock because they were nice enough to donate the prize for this competition.  [shameless-plug:on] Visit my website, TPGaming.com, and try my Arkanoid and Centipede clones.  Hopefully you won't think they suck ;)  The aural side of Twisted Pair Gaming is a DJ who mixes and writes with ShadeFix.  He's not responsible for the amateur tunes you hear now - I take full blame for those ;)  So if you're into Electronica, especially progressive trance, check out ShadeFix on ACIDPlanet. [shameless-plug:off]  There. Now to balance out your GBA's karma cache (0A007700h, I believe), here's plugs for cool stuff that dosn't involve me... [fanboy-mode:on] See MattCurrent.org, Watch Farb-Rausch.de, Hear The Prodigy and The Crystal Method, and Enjoy ChronicLogic.com!  GBAdev.org is great for GBA Dev. Good things are a-comin' at xgamestation.com. PDRoms.de is excellent, of course.  Vivisimo is better than Google (no flames, please ;). And Cleveland Rocks!!!! (ok, sorta...)[fanboy-mode:off]    It seems I've run out of things to say, so have fun with this game, and thanks for playing! This scrolly will repeat in 3.....2.....1.....        ";
//const char scrollyText[] = "the quick brown fox jumps over the lazy dog THE QUICK BROWN FOX JUMPS OVER THE LAZY DOG 01234567890 .. ,, :: (( )) [[ ]] \"\" \'\' ?? !! -- ;; ";
//const char scrollyText[] = "AAABBBCDEYYZZaabbyyz";

////////////////////////////////////////////

void MapImagesToSpritesIntro(void)
{
	spriteImg[spSkipIntroL] = &img_skipIntroL;
	spriteImg[spSkipIntroR] = &img_skipIntroR;
	spriteImg[spEnemy1] = &img_enemyIntro2;
	spriteImg[spEnemy2] = &img_enemyIntro2;
}

void LoadImagesIntro(void)
{
    LoadSpriteImage(&img_au, SPRITE_4BIT);
    LoadSpriteImage(&img_bu, SPRITE_4BIT);
    LoadSpriteImage(&img_cu, SPRITE_4BIT);
    LoadSpriteImage(&img_du, SPRITE_4BIT);
    LoadSpriteImage(&img_eu, SPRITE_4BIT);
    LoadSpriteImage(&img_fu, SPRITE_4BIT);
    LoadSpriteImage(&img_gu, SPRITE_4BIT);
    LoadSpriteImage(&img_hu, SPRITE_4BIT);
    LoadSpriteImage(&img_iu, SPRITE_4BIT);
    LoadSpriteImage(&img_ju, SPRITE_4BIT);
    LoadSpriteImage(&img_ku, SPRITE_4BIT);
    LoadSpriteImage(&img_lu, SPRITE_4BIT);
    LoadSpriteImage(&img_mu, SPRITE_4BIT);
    LoadSpriteImage(&img_nu, SPRITE_4BIT);
	LoadSpriteImage(&img_ou, SPRITE_4BIT);
	LoadSpriteImage(&img_pu, SPRITE_4BIT);
    LoadSpriteImage(&img_qu, SPRITE_4BIT);
    LoadSpriteImage(&img_ru, SPRITE_4BIT);
    LoadSpriteImage(&img_su, SPRITE_4BIT);
    LoadSpriteImage(&img_tu, SPRITE_4BIT);
    LoadSpriteImage(&img_uu, SPRITE_4BIT);
    LoadSpriteImage(&img_vu, SPRITE_4BIT);
    LoadSpriteImage(&img_wu, SPRITE_4BIT);
    LoadSpriteImage(&img_xu, SPRITE_4BIT);
    LoadSpriteImage(&img_yu, SPRITE_4BIT);
    LoadSpriteImage(&img_zu, SPRITE_4BIT);
    
    LoadSpriteImage(&img_al, SPRITE_4BIT);
    LoadSpriteImage(&img_bl, SPRITE_4BIT);
    LoadSpriteImage(&img_cl, SPRITE_4BIT);
    LoadSpriteImage(&img_dl, SPRITE_4BIT);
    LoadSpriteImage(&img_el, SPRITE_4BIT);
    LoadSpriteImage(&img_fl, SPRITE_4BIT);
    LoadSpriteImage(&img_gl, SPRITE_4BIT);
    LoadSpriteImage(&img_hl, SPRITE_4BIT);
    LoadSpriteImage(&img_il, SPRITE_4BIT);
    LoadSpriteImage(&img_jl, SPRITE_4BIT);
    LoadSpriteImage(&img_kl, SPRITE_4BIT);
    LoadSpriteImage(&img_ll, SPRITE_4BIT);
    LoadSpriteImage(&img_ml, SPRITE_4BIT);
    LoadSpriteImage(&img_nl, SPRITE_4BIT);
	LoadSpriteImage(&img_ol, SPRITE_4BIT);
	LoadSpriteImage(&img_pl, SPRITE_4BIT);
    LoadSpriteImage(&img_ql, SPRITE_4BIT);
    LoadSpriteImage(&img_rl, SPRITE_4BIT);
    LoadSpriteImage(&img_sl, SPRITE_4BIT);
    LoadSpriteImage(&img_tl, SPRITE_4BIT);
    LoadSpriteImage(&img_ul, SPRITE_4BIT);
    LoadSpriteImage(&img_vl, SPRITE_4BIT);
    LoadSpriteImage(&img_wl, SPRITE_4BIT);
    LoadSpriteImage(&img_xl, SPRITE_4BIT);
    LoadSpriteImage(&img_yl, SPRITE_4BIT);
    LoadSpriteImage(&img_zl, SPRITE_4BIT);

    LoadSpriteImage(&img_comma, SPRITE_4BIT);
    LoadSpriteImage(&img_space, SPRITE_4BIT);
    LoadSpriteImage(&img_dot, SPRITE_4BIT);
    LoadSpriteImage(&img_dash, SPRITE_4BIT);
    LoadSpriteImage(&img_exclamation, SPRITE_4BIT);
    LoadSpriteImage(&img_parenthL, SPRITE_4BIT);
    LoadSpriteImage(&img_parenthR, SPRITE_4BIT);
    LoadSpriteImage(&img_bracketL, SPRITE_4BIT);
    LoadSpriteImage(&img_bracketR, SPRITE_4BIT);
    LoadSpriteImage(&img_singleQ, SPRITE_4BIT);
    LoadSpriteImage(&img_doubleQ, SPRITE_4BIT);
    LoadSpriteImage(&img_9, SPRITE_4BIT);
    LoadSpriteImage(&img_8, SPRITE_4BIT);
    LoadSpriteImage(&img_7, SPRITE_4BIT);
    LoadSpriteImage(&img_6, SPRITE_4BIT);
    LoadSpriteImage(&img_5, SPRITE_4BIT);
    LoadSpriteImage(&img_4, SPRITE_4BIT);
    LoadSpriteImage(&img_3, SPRITE_4BIT);
    LoadSpriteImage(&img_2, SPRITE_4BIT);
    LoadSpriteImage(&img_1, SPRITE_4BIT);
    LoadSpriteImage(&img_0, SPRITE_4BIT);
    LoadSpriteImage(&img_question, SPRITE_4BIT);
    LoadSpriteImage(&img_colon, SPRITE_4BIT);
    LoadSpriteImage(&img_semicolon, SPRITE_4BIT);

	LoadSpriteImage(&img_skipIntroL, SPRITE_4BIT);
	LoadSpriteImage(&img_skipIntroR, SPRITE_4BIT);
	LoadSpriteImage(&img_enemyIntro, SPRITE_4BIT);
	LoadSpriteImage(&img_enemyIntro2, SPRITE_4BIT);
}

void InitSpritesIntro(void)
{
	  // Do other Sprite Setup
	MapImagesToSpritesIntro();
	LoadImagesIntro();

	  // Call Sprite_InitY() for each sprite
//	Sprite_InitXY(spConsoleFeverL, SPRITE_4BIT,
//		CONSOLEFEVER_INTRO_X, CONSOLEFEVER_INTRO_Y);
//	Sprite_InitXY(spConsoleFeverR, SPRITE_4BIT,
//		CONSOLEFEVER_INTRO_X+img_consoleFeverBoxedL.width, CONSOLEFEVER_INTRO_Y);

	Sprite_InitXY(spSkipIntroL, SPRITE_4BIT,
		PRESSSTARTINTRO_X, PRESSSTARTINTRO_Y);
	Sprite_InitXY(spSkipIntroR, SPRITE_4BIT,
		PRESSSTARTINTRO_X+img_skipIntroL.width, PRESSSTARTINTRO_Y);

	Sprite_InitXY(spEnemy1, SPRITE_4BIT, 2, 130);
	Sprite_InitXY(spEnemy2, SPRITE_4BIT, 122, 130);

	  // Put on Screen
	WaitForVSync();
	Sprite_UpdateOAM(0, endOfIntroSprites);
}

void DoURLDisplay(void)
{
	if(consoleFeverState==0)  // Up
	{
		consoleFeverY -= 0x0008;

		if(consoleFeverY>>4 <= CONSOLEFEVER_INTRO_Y-img_consolefever.height)
		{
			consoleFeverState = 1;  // Show
			consoleFeverDelay=0;
		}
	}
	else if(consoleFeverState==1)  // Show
	{
		consoleFeverDelay++;
				
		if(consoleFeverDelay >= 60)
		{
			consoleFeverState = 2; // Down
		}
	}
	else if(consoleFeverState==2) // Down
	{
		consoleFeverY += 0x0008;

		if(consoleFeverY>>4 >= CONSOLEFEVER_INTRO_Y)
		{
			consoleFeverY = CONSOLEFEVER_INTRO_Y<<4;
			consoleFeverState = 3; // Wait
			consoleFeverDelay=0;
		}
	}
	else // Wait
	{
		consoleFeverDelay++;
				
		if(consoleFeverDelay >= 100)
		{
			consoleFeverState = 0; // Up
			if(blitURL == &img_consolefever)
				blitURL = &img_tpgaming;
			else
				blitURL = &img_consolefever;
		}
	}
}

int LookupFontImg(char inChr)
{
	if(inChr >= 'A' && inChr <= 'Z')
		return 1020 - ((inChr-'A')*4);

	if(inChr >= 'a' && inChr <= 'z')
		return 1020 - (( 1+(inChr-'A')-('a'-'Z') )*4);

	if(inChr >= '0' && inChr <= '9')
		return img_0.charMemIndex + (inChr-'0')*4;

	switch(inChr)
	{
	case ' ':
		return img_space.charMemIndex;

	case '.':
		return img_dot.charMemIndex;

	case ',':
		return img_comma.charMemIndex;

	case ':':
		return img_colon.charMemIndex;

	case '\"':
		return img_doubleQ.charMemIndex;

	case '\'':
		return img_singleQ.charMemIndex;

	case '[':
		return img_bracketL.charMemIndex;

	case ']':
		return img_bracketR.charMemIndex;

	case '(':
		return img_parenthL.charMemIndex;

	case ')':
		return img_parenthR.charMemIndex;

	case '?':
		return img_question.charMemIndex;

	case '!':
		return img_exclamation.charMemIndex;

	case '-':
		return img_dash.charMemIndex;

	case ';':
		return img_semicolon.charMemIndex;

	}
	return img_space.charMemIndex;
}

const char* scrollyTextPtr;
int leftmostSprite;
void InitScrolly(void)
{
	int i;
	for(i=0; i<16; i++)
	{
		spriteImg[i] = &img_space;
		Sprite_InitXY(i, SPRITE_4BIT, i*17, 20);

		lettersYDegree[i] = i*22;
	}

	scrollyTextPtr = &scrollyText[0];
	leftmostSprite=0;
}


void UpdateScrolly(void)
{
	int i;
	s32 cosVal;

	for(i=0; i<16; i++)
	{
		Sprite_Move(i, -4, 0);

		lettersYDegree[i] -= 12;
		if(lettersYDegree[i] < 0)
			lettersYDegree[i] += 360;

		cosVal = COS[lettersYDegree[i]];
		Sprite_SetY(i, 30 + (cosVal>>12));
	}

	if(Sprite_GetX(leftmostSprite) > 0x01FF - 32 &&
		Sprite_GetX(leftmostSprite) < 0x01FF - 16)
	{
		Sprite_SetCharacterOAMOnly(leftmostSprite, LookupFontImg(*scrollyTextPtr));
		Sprite_Move(leftmostSprite, 16*17, 0);

		leftmostSprite++;
		if(leftmostSprite > 15)
			leftmostSprite = 0;

		scrollyTextPtr++;
		if(*scrollyTextPtr == '\0')
			scrollyTextPtr = &scrollyText[0];
	}
}

vu8 enemyChangeDirFlag=0;
vu8 enemyFangFlag=0;
vu8 enemyFangCount=0;
void UpdateIntroEnemy(void)
{
	static s32 x1 = 10<<6;
	static s32 feedbackGoal=10<<6;

	if(enemyChangeDirFlag)
	{
		enemyChangeDirFlag = 0;

		if(feedbackGoal == 86<<6)
			feedbackGoal = 10<<6;
		else
			feedbackGoal = 86<<6;
	}

	if(enemyFangFlag)
	{
		Sprite_SetCharacter(spEnemy1, &img_enemyIntro);
		Sprite_SetCharacter(spEnemy2, &img_enemyIntro);
		enemyFangFlag = 0;
		enemyFangCount = 0;
	}
	else
	{
		enemyFangCount++;
		if(enemyFangCount==2)
		{
			Sprite_SetCharacter(spEnemy1, &img_enemyIntro2);
			Sprite_SetCharacter(spEnemy2, &img_enemyIntro2);
		}
	}

	x1 += 11*((feedbackGoal - x1) >> 4);

	Sprite_SetX(spEnemy1, x1>>6);
	Sprite_SetX(spEnemy2, (x1>>6)+120);
}

void DoIntro(void)
{
	InitPlasma();
	InitColor();
	UpdateColor();
	InitSpritesIntro();
	InitScrolly();

	EnableInterupts(INT_HBLANK);
	REG_BG2CNT |= 0x2040;
	REG_BG2PA = 0x0080;
	REG_BG2PD = 0x0080;

	StartMusic(0); //0=introMelody
	s16 mosaicVal=0;
	do
	{
		UpdatePlasma();
		UpdateScrolly();
		UpdateIntroEnemy();
	//	WaitForVSync();
		Sprite_UpdateOAM(0, 18);
		UpdateColor();
		DoURLDisplay();
		DrawPlasma(consoleFeverY>>4);

		if(consoleFeverState != 4)
		{
			Blit(blitURL, colorLookup[1], colorLookup[0],// 0, 0);
				CONSOLEFEVER_INTRO_X, consoleFeverY>>4);
		}

		if(mosaicFlag == 1)
		{
			mosaicVal = 12;
			mosaicFlag = 0;
		}
		else
		{
			if(mosaicVal > 0)
			{
				mosaicVal-=4;
				if(mosaicVal < 0)
					mosaicVal = 0;
			}
		}
		if(mosaicVal == 0)
			REG_MOSAIC = 0x0000;
		else
			REG_MOSAIC = (12-mosaicVal) | ((12-mosaicVal) << 4);

		Input_Poll();
	} while(!isKeyPressed(KEY_START));

	DisableInterupts(INT_HBLANK);
	StopMusic();

	FillScreen(BG_COLOR_0);

	REG_BG2PA = 0x0100;
	REG_BG2PD = 0x0100;
	REG_BG2X = 0;
	REG_BG2Y = 0;
	REG_MOSAIC = 0x0000;
	REG_BG2CNT &= ~0x0040;

	WaitForVSync();
	Sprite_ClearOAM(0, 128);
	ResetSpriteImageData();
}

s16 plasmaX1Offset;
s16 plasmaX1FrameInc;
u16 plasmaX1[SCREEN_WIDTH/2];

s16 plasmaY1Offset;
s16 plasmaY1FrameInc;
u16 plasmaY1[SCREEN_HEIGHT/2];

s16 plasmaX2Offset;
s16 plasmaX2FrameInc;
u16 plasmaX2[SCREEN_WIDTH/2];

s16 plasmaY2Offset;
s16 plasmaY2FrameInc;
u16 plasmaY2[SCREEN_HEIGHT/2];

u16 plasmaXCombined[SCREEN_WIDTH/2];
u16 plasmaYCombined[SCREEN_HEIGHT/2];

void InitPlasma(void)
{
	plasmaX1Offset = 0;
	plasmaX2Offset = 0;
	plasmaY1Offset = 0;
	plasmaY2Offset = 0;

	plasmaX1FrameInc = 2;
	plasmaX2FrameInc = -6;
	plasmaY1FrameInc = 5;
	plasmaY2FrameInc = -3;
}

void InitColor(void)
{
	colorRotOn = 1;

	colorDelay=0;
	colorLookup[0] = 0x0000;
	colorLookup[1] = 0x7FFF;
	colorR[0] = 16;
	colorG[0] = 8;
	colorB[0] = 0;
	colorR[1] = 16+8;
	colorG[1] = 8+8;
	colorB[1] = 0+8;
	colorRInc[0] = 2;
	colorGInc[0] = 1;
	colorBInc[0] = 3;
	colorRInc[1] = 2;
	colorGInc[1] = 1;
	colorBInc[1] = 3;

	OBJPaletteMem[0x0F] = colorLookup[1];
	OBJPaletteMem[1]	= colorLookup[0];
}

void UpdateColor(void)
{
	int tmpR, tmpG, tmpB;

	if(colorRotOn)
		colorDelay++;

	if(colorDelay==4)
	{
		colorDelay=0;

		colorR[0] += colorRInc[0];
		colorG[0] += colorGInc[0];
		colorB[0] += colorBInc[0];
	
		if(colorR[0] >= 0x0040)
		{
			colorRInc[0] = -colorRInc[0];
			colorR[0] += colorRInc[0]<<1;
		}

		if(colorG[0] >= 0x0040)
		{
			colorGInc[0] = -colorGInc[0];
			colorG[0] += colorGInc[0]<<1;
		}

		if(colorB[0] >= 0x0040)
		{
			colorBInc[0] = -colorBInc[0];
			colorB[0] += colorBInc[0]<<1;
		}
/*
		colorR[1] += colorRInc[1];
		colorG[1] += colorGInc[1];
		colorB[1] += colorBInc[1];
	
		if(colorR[1] >= 0x0040)
		{
			colorRInc[1] = -colorRInc[1];
			colorR[1] += colorRInc[1]<<1;
		}

		if(colorG[1] >= 0x0040)
		{
			colorGInc[1] = -colorGInc[1];
			colorG[1] += colorGInc[1]<<1;
		}

		if(colorB[1] >= 0x0040)
		{
			colorBInc[1] = -colorBInc[1];
			colorB[1] += colorBInc[1]<<1;
		}
*/
		tmpR = ((colorR[0]>>1) & 0x007F);
		tmpG = ((colorG[0]>>1) & 0x007F);
		tmpB = ((colorB[0]>>1) & 0x007F);
		colorLookup[0] = RGB(tmpR, tmpG, tmpB);
		colorLookup[1] = ~colorLookup[0];
//		tmpR = ((colorR[1]>>1) & 0x007F);
//		tmpG = ((colorG[1]>>1) & 0x007F);
//		tmpB = ((colorB[1]>>1) & 0x007F);
//		colorLookup[1] = RGB(tmpR, tmpG, tmpB);
//		colorLookup[1] = RGB((31-tmpR), (31-tmpG), (31-tmpB));
//		colorLookup[1] = RGB(((tmpR + 16) & 0x007F), ((tmpG + 16) & 0x007F), ((tmpB + 16) & 0x007F));

		OBJPaletteMem[0x0F] = colorLookup[1];
		OBJPaletteMem[1]	= colorLookup[0];
	}
}

void UpdatePlasma(void)
{
	plasmaX1Offset += plasmaX1FrameInc;
	if(plasmaX1Offset >= 360)
		plasmaX1Offset = 0;
	else if(plasmaX1Offset < 0)
		plasmaX1Offset = (360/1) - 1;

	plasmaX2Offset += plasmaX2FrameInc;
	if(plasmaX2Offset >= 360)
		plasmaX2Offset = 0;
	else if(plasmaX2Offset < 0)
		plasmaX2Offset = (360/1) - 1;

	plasmaY1Offset += plasmaY1FrameInc;
	if(plasmaY1Offset >= 360)
		plasmaY1Offset = 0;
	else if(plasmaY1Offset < 0)
		plasmaY1Offset = (360/1) - 1;

	plasmaY2Offset += plasmaY2FrameInc;
	if(plasmaY2Offset >= 360)
		plasmaY2Offset = 0;
	else if(plasmaY2Offset < 0)
		plasmaY2Offset = (360/1) - 1;

	  // Update plasmaX1 array
	int i;
	u16* ptr;
	s16 val;
	for(i=0, ptr=plasmaX1; i<SCREEN_WIDTH/2; i++, ptr++)
	{
		*ptr = (u16)COS[plasmaX1Offset+i*2];
	}

	  // Update plasmaX2 array
	for(i=0, ptr=plasmaX2; i<SCREEN_WIDTH/2; i++, ptr++)
	{
		val = i*4;
		if(val >= 360)
			val -= 360;
		*ptr = (u16)COS[plasmaX2Offset+val];
	}

	  // Update plasmaY arrays
	for(i=0, ptr=plasmaY1; i<SCREEN_HEIGHT/2; i++, ptr++)
	{
		*ptr = (u16)COS[plasmaY1Offset+i*2];
		plasmaY2[i] = (u16)COS[plasmaY2Offset+i*4];
	}

	  // Combine
	for(i=0, ptr=plasmaXCombined; i<SCREEN_WIDTH/2; i++, ptr++)
	{
		*ptr = plasmaX1[i] + plasmaX2[i];
	}
	for(i=0, ptr=plasmaYCombined; i<SCREEN_HEIGHT/2; i++, ptr++)
	{
		*ptr = plasmaY1[i] + plasmaY2[i];
	}
}

void DrawPlasma(int height)
{
	int x, y;
//	const u16 qLookup[8] = {	0x0000, 0x7FFF, 0x0000, 0x7FFF,
//								0x0000, 0x0000, 0x0000, 0x7FFF	};
//	const u16 qLookup[8] = {	0x0000, 0x0000, 0x0000, 0x7FFF,
//								0x0000, 0x0000, 0x7FFF, 0x7FFF	};
	const u16 qLookup[8] = {	0, 0, 0, 1,
								0, 0, 1, 1	};
	u16 val;
	u16* rowPtr=screen;

	u16 yCombined;

	  // Y Loop
//	for(y=0; y<SCREEN_HEIGHT/2; y++)
	for(y=0; y<height; y++)
	{
		yCombined = plasmaYCombined[y];

		  // X Loop
		for(x=0; x<SCREEN_WIDTH/2; x++)
		{

			val = plasmaXCombined[x] + yCombined;

			val = (val >> 14) | (((y ^ x) & 0x0001) << 2);
			*rowPtr = colorLookup[qLookup[val]];

			rowPtr++;
		}
//		rowPtr+=(SCREEN_WIDTH/4)*3;
		rowPtr+=SCREEN_WIDTH/2;
	}
}

// END OF FILE ///////////////////
