// My Robot - Enemy Code
// Nick Sabalausky

#include "gba.h"
#include "userdefs.h"
#include "external.h"

  // GBA Headers
#include "gfx.h"
#include "sprite.h"
#include "input.h"
#include "mathluts.h"

  // Game Headers
#include "gamedefs.h"
#include "intro.h"
#include "sound.h"
#include "enemyCode.h"
#include "robotShotCode.h"

#define tileData    ((u16*)0x6000000)  // Tile Data Block #0
#define tileMap     ((u16*)0x600F800)  // Tile Map Block #31

void DoIntro(void)
{
	int i;

	REG_DISPCNT_L = 1 | BG1_ENABLE | OBJ_MAP_1D | OBJ_ENABLE;
	REG_BG1CNT = 0x1F03;  // 256x256 map, tile data block 0, tile map block 31, priority 3 (back)
//	REG_BG1CNT = 0x1F43;  // Same but with Mosaic on
	REG_MOSAIC = 0x0001;

	BGPaletteMem[0]  = 0;//RGB(15, 0, 30);
	BGPaletteMem[1]  = RGB(15, 0, 30);
	BGPaletteMem[15] = 0x7FFF;

	InitPlasma2X();
//	REG_BG2PA = 0x0080;
//	REG_BG2PD = 0x0080;

	do
	{
		UpdatePlasma2X();
		WaitForVSync();
		DrawPlasma2X();

		Input_Poll();
	} while(!isKeyPressed(KEY_START));

//	REG_BG2PA = 0x0100;
//	REG_BG2PD = 0x0100;
}

s16 plasmaX1Offset;
s16 plasmaX1FrameInc;
u16 plasmaX1[SCREEN_WIDTH];

s16 plasmaY1Offset;
s16 plasmaY1FrameInc;
u16 plasmaY1[SCREEN_HEIGHT];

s16 plasmaX2Offset;
s16 plasmaX2FrameInc;
u16 plasmaX2[SCREEN_WIDTH];

s16 plasmaY2Offset;
s16 plasmaY2FrameInc;
u16 plasmaY2[SCREEN_HEIGHT];

void InitPlasma(void)
{
	int i;
	int x, y;
//	u16 val;

	plasmaX1Offset = 0;
	plasmaX2Offset = 0;
	plasmaY1Offset = 0;
	plasmaY2Offset = 0;

	plasmaX1FrameInc = 2;
	plasmaX2FrameInc = -6;
	plasmaY1FrameInc = 5;
	plasmaY2FrameInc = -3;

	// Screen: Pixels 240x160, Tiles 30x20
	for(y=0; y < 12; y++)
	{
		for(x=0; x < 30; x++)
		{
			i = x+(y*32);
			tileMap[i] = i;         // Palette 0, No VFlip or HFlip

			  // Mirrored version
			if(y >= 4)
			{
				tileMap[x+( ((19-y)+4) *32)] = i | 0x0800;  // Palette 0, VFlip, No HFlip
			}
		}
	}
}

void InitPlasma2X(void)
{
	int i;
	int x, y;

	plasmaX1Offset = 0;
	plasmaX2Offset = 0;
	plasmaY1Offset = 0;
	plasmaY2Offset = 0;

	plasmaX1FrameInc = 2;
	plasmaX2FrameInc = -6;
	plasmaY1FrameInc = 5;
	plasmaY2FrameInc = -3;

	// Screen: Pixels 240x160, Tiles 30x20
	for(y=0; y < 12; y++)
	{
		for(x=0; x < 30; x++)
		{
			i = x+(y*32);
			tileMap[i] = i;         // Palette 0, No VFlip or HFlip

			  // Mirrored version
			if(y >= 4)
			{
				tileMap[x+( ((19-y)+4) *32)] = i | 0x0800;  // Palette 0, VFlip, No HFlip
			}
		}
	}
}

void UpdatePlasma(void)
{
	plasmaX1Offset += plasmaX1FrameInc;
	if(plasmaX1Offset >= 360)
		plasmaX1Offset = 0;
	else if(plasmaX1Offset < 0)
		plasmaX1Offset = (360/1) - 1;

	plasmaX2Offset += plasmaX2FrameInc;
	if(plasmaX2Offset >= 360)
		plasmaX2Offset = 0;
	else if(plasmaX2Offset < 0)
		plasmaX2Offset = (360/1) - 1;

	plasmaY1Offset += plasmaY1FrameInc;
	if(plasmaY1Offset >= 360)
		plasmaY1Offset = 0;
	else if(plasmaY1Offset < 0)
		plasmaY1Offset = (360/1) - 1;

	plasmaY2Offset += plasmaY2FrameInc;
	if(plasmaY2Offset >= 360)
		plasmaY2Offset = 0;
	else if(plasmaY2Offset < 0)
		plasmaY2Offset = (360/1) - 1;

	  // Update plasmaX1 array
	int i;
	s16 val;
	for(i=0; i<SCREEN_WIDTH; i++)
	{
		plasmaX1[i] = (u16)(COS[plasmaX1Offset+i]);
	}

	  // Update plasmaX2 array
	for(i=0; i<SCREEN_WIDTH; i++)
	{
		val = i*2;
		if(val >= 360)
			val -= 360;
		plasmaX2[i] = (u16)(COS[plasmaX2Offset+val]);
	}

	  // Update plasmaY arrays
	for(i=0; i<SCREEN_HEIGHT; i++)
	{
		plasmaY1[i] = (u16)(COS[plasmaY1Offset+i]);
		plasmaY2[i] = (u16)(COS[plasmaY2Offset+i*2]);
	}

/*
	  // Update plasmaX1 array
	int i;
	s16 val;
	for(i=0; i<SCREEN_WIDTH; i++)
	{
		plasmaX1[i] = (s16)COS[plasmaX1Offset+i];
	}

	  // Update plasmaX2 array
	for(i=0; i<SCREEN_WIDTH; i++)
	{
		val = i*2;
		if(val >= 360)
			val -= 360;
		plasmaX2[i] = (s16)COS[plasmaX2Offset+val];
	}

	  // Update plasmaY arrays
	for(i=0; i<SCREEN_HEIGHT; i++)
	{
		plasmaY1[i] = (s16)COS[plasmaY1Offset+i];
		plasmaY2[i] = (s16)COS[plasmaY2Offset+i*2];
	}
*/
}
void UpdatePlasma2X(void)
{
	plasmaX1Offset += plasmaX1FrameInc;
	if(plasmaX1Offset >= 360)
		plasmaX1Offset = 0;
	else if(plasmaX1Offset < 0)
		plasmaX1Offset = (360/1) - 1;

	plasmaX2Offset += plasmaX2FrameInc;
	if(plasmaX2Offset >= 360)
		plasmaX2Offset = 0;
	else if(plasmaX2Offset < 0)
		plasmaX2Offset = (360/1) - 1;

	plasmaY1Offset += plasmaY1FrameInc;
	if(plasmaY1Offset >= 360)
		plasmaY1Offset = 0;
	else if(plasmaY1Offset < 0)
		plasmaY1Offset = (360/1) - 1;

	plasmaY2Offset += plasmaY2FrameInc;
	if(plasmaY2Offset >= 360)
		plasmaY2Offset = 0;
	else if(plasmaY2Offset < 0)
		plasmaY2Offset = (360/1) - 1;

	  // Update plasmaX1 array
	int i;
	s16 val;
	for(i=0; i<SCREEN_WIDTH/2; i++)
	{
		plasmaX1[i] = (u16)(COS[plasmaX1Offset+i*2]);
	}

	  // Update plasmaX2 array
	for(i=0; i<SCREEN_WIDTH/2; i++)
	{
		val = i*4;
		if(val >= 360)
			val -= 360;
		plasmaX2[i] = (u16)(COS[plasmaX2Offset+val]);
	}

	  // Update plasmaY arrays
	for(i=0; i<SCREEN_HEIGHT; i++)
	{
		plasmaY1[i] = (u16)(COS[plasmaY1Offset+i]);
		plasmaY2[i] = (u16)(COS[plasmaY2Offset+i*2]);
	}
}
/*
void DrawPlasma(void)
{
	int x, y;
	const u16 qLookup[2] = {0x0000, 0x7FFF};
	u16 val;

	s32 xVal1;
	s32 xVal2;
	s32 yVal1;
	s32 yVal2;

	u16 val1;
	u16 val2;
	u16 val3;
	u16 val4;
	u16* rowPtr=screen;
	u16* rowPtr1=screen;
	u16* rowPtr2=screen+1;
	u16* rowPtr3=screen+SCREEN_WIDTH;
	u16* rowPtr4=screen+SCREEN_WIDTH+1;

	  // Y Loop
	for(y=0; y<SCREEN_HEIGHT; y+=2)
	{
		  // X Loop
		for(x=0; x<SCREEN_WIDTH; x+=2)
		{

			xVal1 = (s32)plasmaX1[x] + (s32)plasmaX2[x];
			xVal2 = (s32)plasmaX1[x+1] + (s32)plasmaX2[x+1];

			yVal1 = (s32)plasmaY1[y] + (s32)plasmaY2[y];
			yVal2 = (s32)plasmaY1[y+1] + (s32)plasmaY2[y+1];

			val1 = (u16)((xVal1 + yVal1)>>0);
			val2 = (u16)((xVal2 + yVal1)>>0);
			val3 = (u16)((xVal1 + yVal2)>>0);
			val4 = (u16)((xVal2 + yVal2)>>0);

			*rowPtr1 = qLookup[val1>>15];
			*rowPtr2 = qLookup[val2>>15];
			*rowPtr3 = qLookup[val3>>15];
			*rowPtr4 = qLookup[val4>>15];

			rowPtr1+=2;
			rowPtr2+=2;
			rowPtr3+=2;
			rowPtr4+=2;
		}
		rowPtr1+=SCREEN_WIDTH;
		rowPtr2+=SCREEN_WIDTH;
		rowPtr3+=SCREEN_WIDTH;
		rowPtr4+=SCREEN_WIDTH;
	}
}
*/

void DrawPlasma(void)
{
	int x, y, tileX;
	int i;
	/*const u16 qLookup[64] = {   0x0000, 0x000F, 0x00F0, 0x00FF,
								0x0F00, 0x0F0F, 0x0FF0, 0x0FFF,
								0xF000, 0xF00F, 0xF0F0, 0xF0FF,
								0xFF00, 0xFF0F, 0xFFF0, 0xFFFF	};*/
	u16 value;
	u16* rowPtr=tileData;

	u16 yCache;

	  // Y Loop
	for(y=0; y<12*8; y++)
	{
		yCache = plasmaY1[y] + plasmaY2[y];

		  // X Loop
		for(tileX=0; tileX<30*8; tileX+=8)
		{
			for(x=0; x<=4; x+=4)
			{
				value = 0;
				for(i=0; i<4; i++)
				{
				value |= ((
					plasmaX1[x+(tileX)+i] +
					plasmaX2[x+(tileX)+i] +
					yCache
					) & 0x8000 ) >> (15-(i<<2));
				}

				value |= value << 1;
				value |= value << 2;

				*rowPtr = value;
//				*rowPtr = qLookup[value];
				rowPtr++;
			}

//			rowPtr += 2*8 - 2;
			rowPtr += 14;
		}
		if((y & 0x0007) == 0x0007)
		{
			rowPtr += (31*(2*8)) - ((30 * (2*8)) - 2);
//			rowPtr += (31*(2*8)) - ((30 * (2*8)) - 4);
//			rowPtr += 496 - 478;
//			rowPtr += 18;
		}
		else
		{
			rowPtr -= (30 * (2*8)) - 2;
//			rowPtr -= ((30 * (2*8)) - 2);
//			rowPtr += 4;
//			rowPtr += 2;
//			rowPtr -= 478;
		}
	}
}

void DrawPlasma2X(void)
{
	int x, y, tileX;
	int i;
//	const u16 qLookup[16] = {   0x0000, 0x000F, 0x00F0, 0x00FF,
//								0x0F00, 0x0F0F, 0x0FF0, 0x0FFF,
//								0xF000, 0xF00F, 0xF0F0, 0xF0FF,
//								0xFF00, 0xFF0F, 0xFFF0, 0xFFFF	};
//	const u16 qLookup[4] =  {   0x0000, 0x00FF,
//								0xFF00, 0xFFFF	};
	u16 value;
	u16* rowPtr=tileData;

	u16 yCache;

	  // Y Loop
	for(y=0; y<12*8; y++)
	{
		yCache = plasmaY1[y] + plasmaY2[y];

		  // X Loop
		for(tileX=0; tileX<30*8; tileX+=8)
		{
			for(x=0; x<=2; x+=2)
			{
//				value = 0;
//				for(i=0; i<2; i++)
//				{
				value = ((
					plasmaX1[x+(tileX)] +
					plasmaX2[x+(tileX)] +
					yCache
					) & 0x8000 ) >> 15;
				value |= ((
					plasmaX1[x+(tileX)+1] +
					plasmaX2[x+(tileX)+1] +
					yCache
					) & 0x8000 ) >> 7;
//				}

				value |= value << 1;
				value |= value << 2;

				*rowPtr = value;
//				*rowPtr = qLookup[value];
				rowPtr++;
			}

//			rowPtr += 2*8 - 2;
			rowPtr += 14;
		}
		if((y & 0x0003) == 0x0003)
		{
//			rowPtr += (31*(2*8)) - ((30 * (2*8)) - 2);
//			rowPtr += (31*(2*8)) - ((30 * (2*8)) - 4);
			rowPtr += 18;
		}
		else
		{
//			rowPtr -= (30 * (2*8)) - 2;
//			rowPtr -= (30 * (2*8)) - 4;
			rowPtr -= 478;
		}
	}
}

// END OF FILE ///////////////////
