// My Robot - Robot Shot - robotShot.c
// Nick Sabalausky

#include "gba.h"
#include "userdefs.h"

  // GBA Headers
#include "gfx.h"
#include "sprite.h"
#include "input.h"

  // Game Headers
#include "gamedefs.h"
#include "sound.h"
#include "enemyCode.h"
#include "robotShotCode.h"

int zapDelay;
int frameCountRobotShot=0;
int shouldCreateRobotShot=0;

extern int robotPos;

void ResetRobotShots(void)
{
	int i;
	for(i=spRobotShotFirst; i <= spRobotShotLast; i++)
	{
		LCD_OFF(i);
	}
	shouldCreateRobotShot=0;
	frameCountRobotShot = 0;
}

extern void DisplayBatteryStat(int numBatteries);
void DoRobotShots(int shouldShoot, int *numBatteries)
{
	static int robotShootPos=0;

	if(shouldShoot && !shouldCreateRobotShot)
	{
		robotShootPos = robotPos;
//		robotShootPos = FindRobotPos();

		// If robot was found
		if(robotShootPos < NUM_ENEMY_COLS)
		{
			shouldCreateRobotShot = 1;
			PlayDown3();
		}
	}

	frameCountRobotShot++;
	if(frameCountRobotShot >= ROBOTSHOT_ANIM_DELAY)
	{
		frameCountRobotShot = 0;

		if(IS_LCD_ON(spZap))
		{
			zapDelay++;
			if(zapDelay==1)
			{
				zapDelay=0;
				LCD_OFF(spZap);
			}
		}


		DoRobotShotUpdate();

		if(shouldCreateRobotShot)
		{
			shouldCreateRobotShot = 0;
			(*numBatteries)--;
			DisplayBatteryStat(*numBatteries);

			CreateRobotShot(robotShootPos);
		}
	}

	DoRobotShotCollide();
}

/*
  // Probably shouldn't be in this file
int FindRobotPos(void)
{
	int i;

	  // Find robot's position
	for(i=0; i < NUM_ENEMY_COLS; i++)
	{
		if(IS_LCD_ON(spRobotFirst + i))
			break;
	}

	return i;
}
*/

void CreateRobotShot(int robotPos)
{
	enumSprites spriteIndex;

	spriteIndex = spRobotShotFirst + robotPos + (NUM_ENEMY_COLS * (NUM_ENEMY_ROWS-1));
	LCD_ON(spriteIndex);
}

void DoRobotShotUpdate(void)
{
	int i;

	  // Top Row
	for(i = spRobotShotFirst; i < spRobotShotFirst+NUM_ENEMY_COLS; i++)
	{
		LCD_OFF(i);
	}

	for(; i <= spRobotShotLast; i++)
	{
		  // If enemy exists at this location
		if(IS_LCD_ON(i))
		{
			LCD_OFF(i);
			LCD_ON(i-NUM_ENEMY_COLS);
		}
	}
}

extern void SetScoreDisplay(int score);
void DoRobotShotCollide(void)
{
	int i;
	  // NUM_ENEMIES is also the number of robot shots
	for(i=0; i < NUM_ENEMIES; i++)
	{
		  // If both the enemy and robot shot in position 'i'
		  // are turned on, then they've collided.
		if( IS_LCD_ON(spEnemyFirst + i) &&
			IS_LCD_ON(spRobotShotFirst + i) )
		{
			PlayDown2();

			LCD_OFF(spEnemyFirst + i);
			LCD_OFF(spRobotShotFirst + i);

			score += SCORE_PER_ENEMYHIT;
			SetScoreDisplay(score);

			LCD_ON(spZap);
			zapDelay = 0;
		}
	}
}

// END OF FILE /////////////////