// GBA Sprite System - sprite.h
// Nick Sabalausky

#ifndef _SPRITE_H_
#define _SPRITE_H_

#define OAM_ATTR0(a)  OAMmem[(a)*4 + 0]
#define OAM_ATTR1(a)  OAMmem[(a)*4 + 1]
#define OAM_ATTR2(a)  OAMmem[(a)*4 + 2]
#define OAM_ATTR3(a)  OAMmem[(a)*4 + 3]

typedef enum _enumSpriteColorDepth {SPRITE_4BIT = 0, SPRITE_8BIT = (1<<13)}
	enumSpriteColorDepth;

#define INITIAL_LASTTAKENCHARDATA 1024
extern s16 lastTakenCharData;

extern OAMEntry sprites[MAX_SPRITES];
extern img *spriteImg[MAX_SPRITES];

  // Position
void inline Sprite_SetX(u16 spriteIndex, u16 x);
void inline Sprite_SetY(u16 spriteIndex, u16 y);
void inline Sprite_SetXY(u16 spriteIndex, u16 x, u16 y);
void Sprite_Move(u16 spriteIndex, u16 dx, u16 dy);
u16 inline Sprite_GetX(u16 spriteIndex);
u16 inline Sprite_GetY(u16 spriteIndex);

  // Horizontal Flip
void inline Sprite_HFlipOn(u16 spriteIndex);
void inline Sprite_HFlipOff(u16 spriteIndex);
void inline Sprite_HFlipToggle(u16 spriteIndex);

  // Vertical Flip
void inline Sprite_VFlipOn(u16 spriteIndex);
void inline Sprite_VFlipOff(u16 spriteIndex);
void inline Sprite_VFlipToggle(u16 spriteIndex);

  // Mosaic
void inline Sprite_MosaicOn(u16 spriteIndex);
void inline Sprite_MosaicOff(u16 spriteIndex);
void inline Sprite_MosaicToggle(u16 spriteIndex);
int  inline Sprite_IsMosaicOn(u16 spriteIndex);

  // Size Double Flag (If a sprite's rotation flag is turned off,
  //   then turning size double on makes the sprite not get drawn)
void inline Sprite_SizeDoubleOn(u16 spriteIndex);
void inline Sprite_SizeDoubleOff(u16 spriteIndex);
void inline Sprite_SizeDoubleToggle(u16 spriteIndex);
int  inline Sprite_IsSizeDoubleOn(u16 spriteIndex);

  // Character Data
void inline Sprite_SetCharacterOAMOnly(u16 spriteIndex, u16 charIndex);
void inline Sprite_SetCharacter(u16 spriteIndex, img *pImg);
void inline Sprite_SetSizeShape(u16 spriteIndex, u16 size, u16 shape);
void inline Sprite_SetColorDepth(u16 spriteIndex, enumSpriteColorDepth colorDepth);
void inline LoadSpriteImage8BitTo(u16 charIndex, img *pImg);
void LoadSpriteImage(img *pImg, enumSpriteColorDepth colorDepth);
void inline ResetSpriteImageData();

void inline LoadSpritePalette8Bit(const u16 *data);
void inline SetSpritePaletteEntry8Bit(u16 index, u16 color);
u16  inline GetSpritePaletteEntry8Bit(u16 index);

void Sprite_UpdateOAM(int oamIndexStart, int numEntries);
void Sprite_ClearOAM(int oamIndexStart, int numEntries);
#define Sprite_Init(a, b) Sprite_InitXY((a), (b), 0, 0);
void Sprite_InitXY(u16 spriteIndex, enumSpriteColorDepth colorDepth, u16 x, u16 y);

#endif
// END OF FILE //////////////////////////
